SystemOrganization addCategory: #SMark!
SystemOrganization addCategory: #'SMark-Tests'!
SystemOrganization addCategory: #'SMark-Classic'!

!WriteStream methodsFor: '*SMark-printing' stamp: 'sm 5/25/2011 14:30'!
<< aCollection
	"we want a readable version of nextPutAll however it may be difficult to fully recreate nextPutAll:
	for all the different types of stream. Rather then simply send to nextPutAll:
	we handle the String (or ByteArray) argument
	as fast as possible - the rest we delegate to putOn: This means that we handle single characters and bytes 
	whereas nextPutAll: is only for sequencable collections.
	.
	Note this may not work in every case that nextPutAll: does subject to extensive testing, 
	but it should work in the important cases"

	| newEnd |
	collection class == aCollection class ifFalse:
		[ aCollection putOn: self. ^ self ].

	newEnd := position + aCollection size.
	newEnd > writeLimit ifTrue:
		[self growTo: newEnd + 10].

	collection replaceFrom: position+1 to: newEnd  with: aCollection startingAt: 1.
	position := newEnd.

! !

!Time class methodsFor: '*SMark-squeak protocol' stamp: 'sm 5/25/2011 14:26'!
milliseconds: currentTime since: lastTime
	"Answer the elapsed time since last recorded in milliseconds.
	Compensate for rollover."

	| delta |
	delta := currentTime - lastTime.
	^ delta < 0
		ifTrue: [SmallInteger maxVal + delta]
		ifFalse: [delta]
! !

TestCase subclass: #SMarkHarnessArgumentParserTest
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
setUp
	parser := SMarkHarnessArgumentParser new.
	parser harness: SMarkHarness.! !

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
testParseArgumentsOnlyBench
	| arguments runnerAndReporter runner reporter runOnly |
	arguments := {#ignored. 'SMarkTestSuite.benchA' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	runOnly := runner suite selectedBenchmarks.
	self assert: 1 equals: runOnly size.
	self assert: (runOnly includes: #benchA).
! !

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
testParseArgumentsOnlySuite
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestSuite' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
! !

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
testParseArgumentsReporterSuiteIterations
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkSimpleStatisticsReporter'. 'SMarkTestSuite'. '13'. }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (reporter isKindOf: SMarkSimpleStatisticsReporter).
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.	! !

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
testParseArgumentsRunnerReporterSuiteIterations
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestRunner'. 'SMarkSimpleStatisticsReporter'. 'SMarkTestSuite'. '13'. }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner isKindOf: SMarkTestRunner).
	self assert: (reporter isKindOf: SMarkSimpleStatisticsReporter).
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.! !

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
testParseArgumentsRunnerSuiteIterations
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestRunner'. 'SMarkTestSuite'. '13'. }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner isKindOf: SMarkTestRunner).
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.! !

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
testParseArgumentsSuiteAndIterations
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestSuite'. '13' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.! !

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
testParseArgumentsSuiteIterationsProcesses
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestSuite'. '13'. '91' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.
	self assert: 91 equals: runner processes.
! !

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
testParseArgumentsSuiteIterationsProcessesProblemSize
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestSuite'. '13'. '91'. 'SSS' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.
	self assert: 91 equals: runner processes.
	self assert: 'SSS' equals: runner problemSize.
! !

!SMarkHarnessArgumentParserTest methodsFor: 'as yet unclassified' stamp: ''!
testParseBenchmarkOrSuite
	| example1 example2 example3 example4 example5 |
	example1 := 'String>>findTokens:'.
	example2 := 'SMarkTestSuite'.
	example3 := { SMarkTestSuite. #benchA }.
	example4 := SMarkTestSuite.
	example5 := 'String.findTokens:'. ". (dot) notation is required to avoid quoting in bash"
	
	self assert: { String. #findTokens: }    equals: (parser parseBenchmarkOrSuite: example1).
	self assert: SMarkTestSuite              equals: (parser parseBenchmarkOrSuite: example2).
	self assert: { SMarkTestSuite. #benchA } equals: (parser parseBenchmarkOrSuite: example3).
	self assert: SMarkTestSuite              equals: (parser parseBenchmarkOrSuite: example4).
	self assert: { String. #findTokens: }    equals: (parser parseBenchmarkOrSuite: example5).
	
	"What to do when the input is not a symbol, array, or class?"
	
	"What to do when the class does not exist"
	self should: [parser parseBenchmarkOrSuite: 'FooBarClassThatDoesNotExist'] raise: Error.! !

TestCase subclass: #SMarkHarnessClassTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkHarnessClassTest methodsFor: 'tests' stamp: ''!
testShouldShowUsage
	| example1 example2 example3 example4 |
	example1 := { #ignored. '--help' }.
	example2 := { #ignored. }.
	example3 := { #ignored. '-?'}.
	example4 := { #ignored. 1. 2. }.
	
	self assert: (SMarkHarness shouldShowUsage: example1).
	self assert: (SMarkHarness shouldShowUsage: example2).
	self assert: (SMarkHarness shouldShowUsage: example3).
	self deny:   (SMarkHarness shouldShowUsage: example4).
! !

TestCase subclass: #SMarkReporterTest
	instanceVariableNames: 'runner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkReporterTest methodsFor: 'as yet unclassified' stamp: ''!
setUp
	runner := SMarkTestRunnerSuiteForAutosizing run.! !

!SMarkReporterTest methodsFor: 'as yet unclassified' stamp: ''!
testContentOfStream
	"Hm, how to test that robustly?"
	| reporter stream out |
	stream := TextStream on: String new.
	reporter := SMarkReporter reportFor: runner on: stream.
	
	out := stream contents.
	
	self assert: (out includesSubString: 'Small').
	self assert: (out includesSubString: 'Big').
	self assert: (out includesSubString: 'Small').! !

!SMarkReporterTest methodsFor: 'as yet unclassified' stamp: ''!
testConvenienceReportOnRunner
	| stream out |
	stream := TextStream on: String new.
	
	"That should just work, cannot really check whether it is displayed anywhere."
	SMarkTestRunnerSuiteForAutosizing run report.
	
	"Check whether it reported as expected"
	SMarkTestRunnerSuiteForAutosizing run reportOn: stream.
	
	out := stream contents.
	
	self assert: (out includesSubString: 'Small').
	self assert: (out includesSubString: 'Big').
	self assert: (out includesSubString: 'Small').! !

TestCase subclass: #SMarkResultTest
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkResultTest commentStamp: 'StefanMarr 3/18/2011 23:29' prior: 0!
This test specifies the data stored in a result object.
The result object does not actually provide behavior on its own.!

!SMarkResultTest methodsFor: 'as yet unclassified' stamp: ''!
setUp
	| runner |
	runner := SMarkTestRunner execute: SMarkTestRunnerSuite new.
	result := (runner results at: #DoNothing) at: 1.! !

!SMarkResultTest methodsFor: 'as yet unclassified' stamp: ''!
testBenchmarkName
	self assert: (result benchmarkName isKindOf: String)! !

!SMarkResultTest methodsFor: 'as yet unclassified' stamp: ''!
testCriteria
	self assert: (result criteria respondsTo: #at:).
	self assert: (result criteria size > 0).! !

!SMarkResultTest methodsFor: 'as yet unclassified' stamp: ''!
testSuite
	self assert: result suite notNil! !

!SMarkResultTest methodsFor: 'as yet unclassified' stamp: ''!
testTotal
	self assert: (result total isKindOf: Number)! !

TestCase subclass: #SMarkRunnerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

SMarkRunnerTest subclass: #SMarkAutosizeRunnerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkAutosizeRunnerTest class methodsFor: 'as yet unclassified' stamp: ''!
shouldInheritSelectors
	^ true! !

!SMarkAutosizeRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
runnerClass
	^ SMarkTestAutosizeRunner! !

!SMarkAutosizeRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testAutosizing
	"Ensure that the runtime is automatically sized up to a predefined value"
	| runner results big small runTime benchmarkSum |
	
	runner := self runnerClass new.
	
	"sanity check for target time"
	"Not checked since we reduced the targetTime for testing:
	 self assert: self runnerClass defaultTargetTime equals: runner targetTime."
	self assert: (self runnerClass defaultTargetTime > 250).
	
	runner iterations: 3.
	  
	runner suite: SMarkTestRunnerSuiteForAutosizing new.
	
	"Execute the benchmarks and take the overall time it takes"
	runTime := Time millisecondClockValue.
	results := runner execute.
	runTime := Time millisecondClockValue - runTime.
	
	"There should be three benchmarks"
	self assert: 3 equals: results size.
	self assert: (results includesKey: #baseBenchmark).
	big := results at: #Big.
	small := results at: #Small.
	
	self assert: (big respondsTo: #at:).
	self assert: 3 equals: big size.
	
	self assert: (runner targetTime <= (big at: 1) total).
	self assert: (runner targetTime <= (small at: 1) total).
	
	"Calculate the sum of all measurements"
	benchmarkSum := big inject: 0 into: [:sum :result| sum + result total].
	benchmarkSum := small inject: benchmarkSum into: [:sum :result| sum + result total].
	
	"And now make sure that we not try to find the problem size for every but only the first run"
	self assert: (benchmarkSum * 2) > runTime.
	
! !

!SMarkAutosizeRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testAutosizingBaseBenchmark
	"Ensure that baseBenchmarks are not delivering any data since I do not see how that could be done nicely"
	| runner results |
	runner := self runnerClass new.
	runner iterations: 3.
	runner suite: SMarkTestRunnerSuiteForAutosizing new.
	
	"Execute the benchmarks and take the overall time it takes"
	results := runner execute.
	
	"There should be two benchmarks"
	self assert: 3 equals: results size.
	self assert: 0 equals: ((results at: #baseBenchmark) at: 1) total.
	
! !

!SMarkAutosizeRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testBaseBenchmark
	"Different from base test: Ensure the base benchmark is NOT executed when available"
	
	| runner |
	runner := self runnerClass execute: SMarkTestSuiteWithBaseBenchmark new.

	self deny: runner baseBenchmarkExecuted  ! !

!SMarkAutosizeRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testExecute
	"This test is slightly different from the one in the base class"
	| runner results performedBenchmarks |
	runner := self runnerClass new.
	runner suite: SMarkTestRunnerSuiteForPerfromBenchmark new.
	results := runner execute.
	
	performedBenchmarks := runner performedBenchmarks.
		
	self assert: 4 equals: performedBenchmarks size.				"Different from base test"
	self deny: (performedBenchmarks includes: #baseBenchmark).	"Different from base test"
	
	self assert: (results respondsTo: #at:).
	
	"But still we get a result" "Different from base test"
	self assert: (((results at: #baseBenchmark) at: 1) isKindOf: SMarkResult).! !

!SMarkAutosizeRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testIterations
	| runner |
	"This test is different then the one in the base class, it is not direct, but just looks at the results"
	
	" simple scenario should default to 1 run for now "
	runner := self runnerClass execute: SMarkTestRunnerSuite new.
	
	self assert: 1 equals: (runner results at: #NotifyRunner) size.
	
	" but we can also be more fancy and set the number of iterations "
	runner := self runnerClass new.
	runner suite: SMarkTestRunnerSuite new.
	
	"first check again that the default is as expected"
	self assert: 1 equals: runner iterations.  
	
	runner iterations: 10.
	self assert: runner iterations equals: 10.
	
	runner execute.
	
	self assert: 10 equals: (runner results at: #NotifyRunner) size.! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
runnerClass
	^ SMarkTestRunner! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testBaseBenchmark
	"Ensure the base benchmark is executed when available"
	
	| runner |
	runner := self runnerClass execute: SMarkTestSuiteWithBaseBenchmark new.

	self assert: runner baseBenchmarkExecuted  ! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testBaseBenchmarkNone
	"Ensure that not having a base benchmark does not pose a problem"

	| runner |
	runner := self runnerClass execute: SMarkTestSuite new.

	self deny: runner baseBenchmarkExecuted  ! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testBenchmarkWithMultipleCriteraTimers
	"Check whether the benchmark benchWithMultipleCriteria, really results in different unrelated timings"
	| runner result |
	runner := self runnerClass execute: SMarkTestSuite new.
	
	result := (runner results at: #WithMultipleCriteria) at: 1.
	self assert: (result isKindOf: SMarkResult).
	
	self assert: result benchmarkName equals: #WithMultipleCriteria.
	self assert: (result total >= 50).
	
	self assert: (result criteria notNil).
	self assert: (result criteria respondsTo: #at:).
	
	self assert: ((result criteria at: 't1') totalTime >= 10).
	self assert: ((result criteria at: 't2') totalTime >= 25).
	self assert: ((result criteria at: 't3') totalTime >= 50).
	! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testClassExecute
	| runner |
	runner := self runnerClass execute: SMarkTestSuiteWithBaseBenchmark new.
	
	self assert: (runner isKindOf: self runnerClass).! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testExecute
	"Ensure all benchmarks are run, including the baseBenchmark and the results are returned"
	| runner results performedBenchmarks |
	runner := self runnerClass new.
	runner suite: SMarkTestRunnerSuiteForPerfromBenchmark new.
	results := runner execute.
	
	performedBenchmarks := runner performedBenchmarks.
		
	self assert: 5 equals: performedBenchmarks size.
	self assert: (performedBenchmarks includes: #baseBenchmark).
	
	self assert: (results respondsTo: #at:).
	self assert: (((results at: #baseBenchmark) at: 1) isKindOf: SMarkResult).! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testIterations
	| runner |
	" simple scenario should default to 1 run for now "
	runner := self runnerClass execute: SMarkTestRunnerSuite new.
	
	self assert: runner countWasExecuted equals: 1.
	
	" but we can also be more fancy and set the number of iterations "
	runner := self runnerClass new.
	runner suite: SMarkTestRunnerSuite new.
	
	"first check again that the default is as expected"
	self assert: runner iterations equals: 1.  
	
	runner iterations: 10.
	self assert: 10 equals: runner iterations.
	
	runner execute.
	
	self assert: 10 equals: runner countWasExecuted.! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testPerformBenchmark
	"Ensure that performBenchmark executes all the defined #bench methods, but for instance not #Bench"
	
	| runner performedBenchmarks |
	runner := self runnerClass execute: SMarkTestRunnerSuiteForPerfromBenchmark new.
	performedBenchmarks := runner performedBenchmarks.

	self assert: (performedBenchmarks includes: #benchAfsdfsdfsdf).
	self assert: (performedBenchmarks includes: #benchBsdfasereSDSfsdfsdfs).
	self assert: (performedBenchmarks includes: #benchC).
	self assert: (performedBenchmarks includes: #benchZZ).

	self deny: (performedBenchmarks includes: #BenchShouldNotBeExecuted).! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testResultStructure
	| runner results resultA resultB suite |
	suite := SMarkTestSuite new.
	runner := self runnerClass execute: suite.
	results := runner results.
	
	"Don't care whether it is a dictionary, but at least it should present us something that maps benchmark names to result objects"
	self assert: (results respondsTo: #at:).

	resultA := (results at: #A) at: 1.
	resultB := (results at: #B) at: 1.


	self assert: (resultA isKindOf: SMarkResult).
	self assert: (resultB isKindOf: SMarkResult).
	self assert: (((results at: #C) at: 1) isKindOf: SMarkResult).
	
	"Make sure some properties of the result objects is ensured"
	self assert: resultA total > 0.
	self assert: #A equals: resultA benchmarkName.
	self assert: suite equals: resultA suite.
	self assert: 1 equals: resultA criteria size.
	
	self assert: resultB total > 0.
	self assert: #B equals: resultB benchmarkName.
	self assert: suite equals: resultB suite.
	self assert: 1 equals: resultB criteria size.
	"Testing for multi-criteria results somewhere else"! !

!SMarkRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testTimer
	"Simple test whether a timer is created and it is from the expected class.
	 This test is very basic, other tests like testBenchmarkWithMultipleCirteria* need to cover the rest"
	| runner timer |
	runner := self runnerClass new.
	timer := runner createTimer: #foo.

	self assert: timer name equals: #foo.
	self assert: (timer isKindOf: self runnerClass defaultTimer).  ! !

SMarkRunnerTest subclass: #SMarkWeakScalingRunnerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkWeakScalingRunnerTest class methodsFor: 'as yet unclassified' stamp: ''!
shouldInheritSelectors
	^ true! !

!SMarkWeakScalingRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
runnerClass
	^ SMarkTestWeakScalingRunner! !

!SMarkWeakScalingRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testInnerIterations
	"Test the semantics of the inner loop parameter"
	| runner results |
	
	"Setup"
	runner := self runnerClass new.
	runner iterations: 1.
	runner innerIterations: 1.  
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 10 equals: runner countWasExecuted.
	self assert: 1  equals: (results at: #NotifyRunner) size.
	
	runner := self runnerClass new.
	runner iterations: 2.
	runner innerIterations: 1.  
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 20 equals: runner countWasExecuted.
	self assert: 2  equals: (results at: #NotifyRunner) size.
	
	
	runner := self runnerClass new.
	runner iterations: 2.
	runner innerIterations: 4.  
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: (20 * 4) equals: runner countWasExecuted.
	self assert: 2  equals: (results at: #NotifyRunner) size.
	
	runner := self runnerClass new.
	runner iterations: 4.
	runner innerIterations: 7.  
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: (40 * 7) equals: runner countWasExecuted.
	self assert: 4           equals: (results at: #NotifyRunner) size.! !

!SMarkWeakScalingRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testWeakScaling3Processes
	"test with small number of processes and varying number of iterations"
	| runner results |
	
	"Setup for 2 iteration, and 3 process"
	runner := self runnerClass new.
	runner iterations: 2.
	runner processes: 3.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 6 equals: runner countWasExecuted.
	self assert: 2 equals: (results at: #NotifyRunner) size.! !

!SMarkWeakScalingRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testWeakScalingFewProcesses
	"test with small number of processes and varying number of iterations"
	| runner results |
	
	"Setup for 1 iteration, and 1 process"
	runner := self runnerClass new.
	runner iterations: 1.
	runner processes: 1.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 1 equals: runner countWasExecuted.
	! !

!SMarkWeakScalingRunnerTest methodsFor: 'as yet unclassified' stamp: ''!
testWeakScalingManyProcesses
	"test with large number of processes and varying number of iterations"
	| runner results |
	
	"Setup"
	runner := self runnerClass new.
	runner iterations: 1.
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 10 equals: runner countWasExecuted.
	self assert: 1  equals: (results at: #NotifyRunner) size.
	
	runner := self runnerClass new.
	runner iterations: 2.
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 20 equals: runner countWasExecuted.
	self assert: 2  equals: (results at: #NotifyRunner) size.
	
	runner := self runnerClass new.
	runner iterations: 4.
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 40 equals: runner countWasExecuted.
	self assert: 4  equals: (results at: #NotifyRunner) size.! !

TestCase subclass: #SMarkSuiteTest
	instanceVariableNames: 'didSetUpBenchB didTearDownBenchC'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testBenchmarkNameForSelector
	| suite |
	suite := SMarkSuite new.
	
	self assert: (suite benchmarkNameForSelector: #benchSomething) equals: #Something.
	self assert: (suite benchmarkNameForSelector: #benchsomeThingElse) equals: #someThingElse.
	
	self assert: (suite benchmarkNameForSelector: #nobenchSomeThing) equals: #nobenchSomeThing.
	self assert: (suite benchmarkNameForSelector: #BenchSomeThing) equals: #BenchSomeThing.! !

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testClassRun
	| runner |
	
	runner := SMarkTestSuite run.
	self assert: (runner isKindOf: SMarkTestSuite defaultRunner).
	self deny: runner results isNil    ! !

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testCleanUpInstanceVariables
	| suite |
	suite := SMarkTestSuite new.
	suite runner: #dummy.
	
	self assert: suite runner	equals: #dummy.
	self assert: suite a			equals: 1.
	self assert: suite c			equals: #three.
				
	suite cleanUpInstanceVariables.
	
	self assert: suite runner	equals: #dummy.
	self assert: suite a			equals: nil.
	self assert: suite b			equals: nil.
	self assert: suite c			equals: nil.
	self assert: suite d			equals: nil.
! !

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testCustomSetUp
	| suite runner |
	
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite.  
	
	self assert: runner hasSetUpBenchB! !

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testCustomTearDown
	| suite runner |
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite.  
	
	self assert: runner hasTearDownBenchC.! !

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testRunOnly
	self assert: false description: 'not yet implemented'.! !

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testSetUp
	| suite runner |
	suite := SMarkTestSuite new.
	
	self assert: suite a			equals: 1.
	self assert: suite b			equals: 2.
	self assert: suite c			equals: #three.
	
	runner := SMarkTestRunner execute: suite.
	
	"Make sure they are reset after each run,
	 well we test just whether they are reset after the final run"
	self assert: suite a			equals: nil.
	self assert: suite b			equals: nil.
	self assert: suite c			equals: nil.
	self assert: suite d			equals: nil.

	"Make sure setUp was called 4 times."
	
	self assert: 4 equals: runner setUpCount.! !

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testSetupAndTeaddownCalledForEveryBenchForEveryIteration
	| suite runner |

	"Setup needs to be executed for every benchmark on every iteration.
	 Test with iterations: 1"
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite with: 1.

	self assert: 4 equals: runner setUpCount.
	self assert: 4 equals: runner tearDownCount.
	
	"Setup needs to be executed for every benchmark on every iteration.
	 Test with iterations: 4"
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite with: 4.

	self assert: (4 * 4) equals: runner setUpCount.
	self assert: (4 * 4) equals: runner tearDownCount.
	
	"Test with iterations: 10"
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite with: 10.
	self assert: (4 * 10) equals: runner setUpCount.
	self assert: (4 * 10) equals: runner tearDownCount.! !

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testShouldRunSelector
	| suite |
	suite := SMarkSuite new.
	
	self assert: (suite shouldRunSelector: #benchSomething).
	self assert: (suite shouldRunSelector: #benchsomeThingElse).
	
	self deny: (suite shouldRunSelector: #nobenchSomeThing).
	self deny: (suite shouldRunSelector: #BenchSomeThing).! !

!SMarkSuiteTest methodsFor: 'as yet unclassified' stamp: ''!
testTearDown
	| suite runner |
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite.  
	
	self assert: 4 equals: runner tearDownCount.! !

TestCase subclass: #SMarkTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTest commentStamp: 'StefanMarr 3/12/2011 21:57' prior: 0!
A general test to ensure the overall framework works as expected.!

!SMarkTest methodsFor: 'running' stamp: ''!
setUp
	"Necessary for Squeak 4.2 compatibility, since method anotations are not supported in the base image."
	(self respondsTo:  #timeout:) ifTrue: [
		self timeout: 300.
	].! !

!SMarkTest methodsFor: 'tests' stamp: ''!
testAllBenchmarksForReturningThemSelf
	"This is a regression test to ensure that the old behavior worked in the first place"
	| allSuites |
	allSuites := SMarkSuite subclasses.
	
	allSuites do: [:suite |
		suite isAbstractClass ifFalse: [
			| runner results |
			Transcript show: (suite class name asString); cr.
			runner := suite run.
			results := runner results.
			self assert: (runner isKindOf: suite defaultRunner).
			self assert: 1 equals: runner iterations.
			self assert: (results size > 0).
		].
	].! !

!SMarkTest methodsFor: 'tests' stamp: ''!
testBasicRun
	| runner results benchResult result |
	
	runner := PBFloat run.
	results := runner results.
	result := (results at: #Add).
	
	self assert: (result isKindOf: Collection).
	self assert: result size equals: 1.
	
	benchResult := result at: 1.
	self assert: (benchResult isKindOf: SMarkResult).
	self assert: (benchResult total notNil).
	! !

TestCase subclass: #SMarkTimerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTimerTest methodsFor: 'as yet unclassified' stamp: ''!
testName
	| timer |
	timer := SMarkTimer new.
	
	self assert: timer name equals: nil.
	
	timer name: #foo.
	
	self assert: timer name equals: #foo.! !

!SMarkTimerTest methodsFor: 'as yet unclassified' stamp: ''!
testReset
	| timer |
	timer := SMarkTimer new.
	
	timer start.
	(Delay forMilliseconds: 10) wait.
	timer stop.
	
	self assert: (timer totalTime >= 10).
	
	timer reset.
	
	self assert: timer totalTime equals: 0.! !

!SMarkTimerTest methodsFor: 'as yet unclassified' stamp: ''!
testStartStop
	| timer |
	timer := SMarkTimer new.
	
	self assert: timer totalTime equals: 0.
	
	timer start.
	
	(Delay forMilliseconds: 10) wait.
	
	timer stop.
	
	self assert: (timer totalTime >= 10).! !

!SMarkTimerTest methodsFor: 'as yet unclassified' stamp: ''!
testTotalTimeWithResume
	| timer |
	timer := SMarkTimer new.
	self assert: timer totalTime equals: 0.
	
	timer start.
	(Delay forMilliseconds: 10) wait.
	timer stop.
	
	self assert: (timer totalTime >= 10).
	
	timer start.
	(Delay forMilliseconds: 10) wait.
	timer stop.
	
	self assert: (timer totalTime >= 20).
	
	timer start.
	(Delay forMilliseconds: 10) wait.
	timer stop.
	
	self assert: (timer totalTime >= 30).! !

Class subclass: #SMarkCompilerTargetClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Classic'!

!SMarkCompilerTargetClass methodsFor: 'compiling' stamp: ''!
acceptsLoggingOfCompilation
	"Answer whether the receiver's method submisions and class defintions should be logged to the changes file and to the current change set."

	^ false! !

!SMarkCompilerTargetClass methodsFor: 'accessing method dictionary' stamp: ''!
addAndClassifySelector: selector withMethod: compiledMethod inProtocol: aCategory notifying: requestor
	| priorMethodOrNil oldProtocol newProtocol |
	priorMethodOrNil := self compiledMethodAt: selector ifAbsent: [nil].
	self addSelectorSilently: selector withMethod: compiledMethod.
	oldProtocol := self organization categoryOfElement: selector.
	SystemChangeNotifier uniqueInstance 
		doSilently: [self organization classify: selector under: aCategory].
	newProtocol := self organization categoryOfElement: selector.! !

!SMarkCompilerTargetClass methodsFor: 'initialization' stamp: 'sm 1/17/2011 09:38'!
initialize
	super initialize.
	self methodDictionary: MethodDictionary new.! !

Object subclass: #SMarkHarness
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkHarness commentStamp: 'StefanMarr 3/12/2011 21:12' prior: 0!
A benchmark harness stears the execution and reporting of benchmarks.
For that purpose, it will use a designated benchmark runner to do the execution and a benchmark reporter to output the results.
The benchmark harness is also parameterized by the benchmark suites that are to be executed.!

SMarkHarness subclass: #ReBenchHarness
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!ReBenchHarness commentStamp: 'StefanMarr 5/16/2011 09:10' prior: 0!
The ReBenchHarness is optimized for use from the command-line.
It is especially meant to be used together with ReBench, a tool to execute and document benchmarks reproducibly.

See: https://github.com/smarr/ReBench#readme!

!ReBenchHarness class methodsFor: 'defaults' stamp: ''!
defaultReporter
	^ ReBenchReporter! !

!ReBenchHarness class methodsFor: 'defaults' stamp: 'sm 5/25/2011 17:06'!
defaultRunner
	^ SMarkWeakScalingRunner! !

!ReBenchHarness class methodsFor: 'helper' stamp: ''!
parseArguments: arguments
	| parser |
	parser := ReBenchHarnessArgumentParser new.
	parser harness: self.  
	^ parser parse: arguments.! !

!ReBenchHarness class methodsFor: 'helper' stamp: 'sm 5/25/2011 17:20'!
usageBenchmarkParameters: usage
	^ usage,		' processes          optional, number of processes/threads used by the benchmarks', String crlf,
				' inner-iterations   optional, number of iterations done by a single process', String crlf,
				' problemSize        optional, depending on benchmark for instance size of used data set', String crlf.
				! !

!ReBenchHarness class methodsFor: 'helper' stamp: 'sm 5/25/2011 17:20'!
usageHeader
	| usage |
	usage := 'SMark Benchmark Framework, version: ', self version, String crlf.
	usage := usage, String crlf.
	usage := usage, 'Usage: <vm+image> ', self name,
				' <suiteOrBenchmark> [processes [inner-iterations [problemSize]]]', String crlf.
	usage := usage, String crlf.
	
	usage := usage, '  This harness is used for weak-scalling benchmarks.', String crlf.
	usage := usage, '  Use the SMarkHarness for more general settings, it offers more options.', String crlf.
	
	usage := usage, String crlf.
	^ usage! !

!ReBenchHarness class methodsFor: 'helper' stamp: ''!
usageReporter: usage
	"Will rely on default, which is good for ReBench, so do not advertise option."
	^ usage! !

!ReBenchHarness class methodsFor: 'helper' stamp: ''!
usageRunner: usage
	"Will rely on default, which is good for ReBench, so do not advertise option."
	^ usage! !

!SMarkHarness class methodsFor: 'defaults' stamp: ''!
defaultOutputDestination
	^ Smalltalk at:       #ScriptConsole
	            ifAbsent: [SMarkReporter defaultOutputDestination]! !

!SMarkHarness class methodsFor: 'defaults' stamp: ''!
defaultReporter
	^ SMarkReporter defaultReporter! !

!SMarkHarness class methodsFor: 'defaults' stamp: ''!
defaultRunner
	^ SMarkSuite defaultRunner! !

!SMarkHarness class methodsFor: 'benchmarking' stamp: 'sm 5/25/2011 17:41'!
execute: runner andReport: reporter
	runner reportConfiguration: self defaultOutputDestination.
	runner execute.
	reporter runner: runner.
	reporter outputStream: self defaultOutputDestination.
	reporter report.! !

!SMarkHarness class methodsFor: 'benchmarking' stamp: ''!
execute: aBenchmarkOrSuite using: aRunnerClass andReport: withAReporterClass
	| parsedBenchmarkOrSuite runner reporter |
	
	parsedBenchmarkOrSuite := self parseBenchmarkOrSuite: aBenchmarkOrSuite.
	
	runner := aRunnerClass new.
	reporter := withAReporterClass new.
	self instructRunner: runner with: parsedBenchmarkOrSuite.
	self execute: runner andReport: reporter.   ! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
parseArguments: arguments
	| parser |
	parser := SMarkHarnessArgumentParser new.
	^ parser parse: arguments.! !

!SMarkHarness class methodsFor: 'script entry' stamp: ''!
run: arguments
	"Execcuted from the command line using something similar to
	 ./vm my.image SMarkHarness SMarkRunner SMarkReporter SMarkLoops\>\>benchIntLoop 1 1 5
	 ./vm my.image SMarkHarness SMarkRunner SMarkReporter SMarkLoops.benchIntLoop 1 1 5"
	| runner reporter runnerAndReporter |
	
	(self shouldShowUsage: arguments)
		ifTrue: [
			self usage.
			^ self.
		].
	
	runnerAndReporter := self parseArguments: arguments.
	runner := runnerAndReporter first.
	reporter := runnerAndReporter second.  
	
	self execute: runner andReport: reporter. ! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
shouldShowUsage: arguments
	
	arguments size < 2 ifTrue: [^ true ].
	
	^ arguments anySatisfy: [:elem | (elem = '--help') or: [elem = '-?'] ].  ! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
usage
	| usage |
	"Example usage: SMarkHarness SMarkRunner SMarkReporter SMarkLoops.benchIntLoop 1 1 5"
	
	usage := self usageHeader.
	
	usage := usage, 'Arguments:', String crlf.
	usage := self usageRunner:   usage.
	usage := self usageReporter: usage.
	usage := usage, ' suiteOrBenchmark   required, either a SMarkSuite with benchmarks,', String crlf.
	usage := usage, '                              or a benchmark denoted by Suite.benchName', String crlf.
	usage := self usageBenchmarkParameters: usage.
	
	ScriptConsole print: usage.! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
usageBenchmarkParameters: usage
	^ usage,	' iterations         optional, number of times the benchmarks are repeated', String crlf,
				' processes          optional, number of processes/threads used by the benchmarks', String crlf,
				' problemSize        optional, depending on benchmark for instance number of', String crlf,
				'                              inner iterations or size of used data set', String crlf.
! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
usageHeader
	| usage |
	usage := 'SMark Benchmark Framework, version: ', self version, String crlf.
	usage := usage, String crlf.
	usage := usage, 'Usage: <vm+image> ', self name, ' [runner] [reporter] <suiteOrBenchmark>', String crlf.
	usage := usage, '                               [iterations [processes [problemSize]]]', String crlf.
	usage := usage, String crlf.
	^ usage! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
usageReporter: usage
	^ usage,	' reporter           optional, a SMarkReporter class that processes', String crlf,
				'                              and displays the results', String crlf.
	! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
usageRunner: usage
	^ usage, ' runner             optional, a SMarkRunner class that executes the benchmarks', String crlf.! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
version
	(Smalltalk classNamed: #ConfigurationOfBenchmarking)
		ifNotNilDo: [:cfg |
			^ cfg project currentVersion versionNumber asString.
		].
	  
	(Smalltalk classNamed: #MCPackage)
		ifNotNilDo: [:mcp |
			| package |
			package := mcp named: 'SMark'.
			package hasWorkingCopy ifTrue: [
				^ package workingCopy ancestors first name.
			].
		].
	
	^ ''.! !

Object subclass: #SMarkHarnessArgumentParser
	instanceVariableNames: 'runner reporter suiteOrBenchmark iterations processes problemSize i current numParams currentObj arguments suite specificBenchmark suiteClass harness'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

SMarkHarnessArgumentParser subclass: #ReBenchHarnessArgumentParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!ReBenchHarnessArgumentParser methodsFor: 'argument parsing' stamp: 'sm 5/25/2011 17:17'!
determineBenchmarkParametersFromArguments
	i := i + 1.
	i <= numParams ifTrue: [
		processes := (arguments at: i) asInteger.
		i := i + 1.
		i <= numParams ifTrue: [
			iterations := (arguments at: i) asInteger.
			i := i + 1.
			i <= numParams ifTrue: [
				problemSize := arguments at: i.
			]
		]
	].! !

!ReBenchHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineReporter
	reporter := harness defaultReporter new.! !

!ReBenchHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineRunner
	runner := harness defaultRunner new.! !

!ReBenchHarnessArgumentParser methodsFor: 'helper' stamp: 'sm 5/25/2011 17:31'!
instructRunner
	super instructRunner.
	
	runner iterations: runner class defaultNumberOfIterations.
	runner innerIterations: iterations.! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: 'sm 5/25/2011 17:38'!
determineBenchmarkParameters
	"Initialize with defaults, will be overwritten in case
	 it is specified."
	iterations := runner class defaultNumberOfIterations.
	processes  := runner class defaultNumberOfProcesses.
	problemSize:= suiteClass defaultProblemSize.
	
	self determineBenchmarkParametersFromArguments.! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineBenchmarkParametersFromArguments
	i := i + 1.
	i <= numParams ifTrue: [
		iterations := (arguments at: i) asInteger.
		i := i + 1.
		i <= numParams ifTrue: [
			processes := (arguments at: i) asInteger.
			i := i + 1.
			i <= numParams ifTrue: [
				problemSize := arguments at: i.
			]
		]
	].! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineReporter
	(currentObj isKindOf: SMarkReporter)
		ifFalse: [ reporter := harness defaultReporter new. ]
		ifTrue:  [ reporter := currentObj.
			i := i + 1.
			i <= numParams ifTrue: [
				current := arguments at: i.
			]
		].! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineRunner
	(currentObj isKindOf: SMarkRunner)
		ifFalse: [ runner := harness defaultRunner new. ]
		ifTrue:  [ runner := currentObj.
			i := i + 1.
			i <= numParams ifTrue: [
				current := arguments at: i.
				currentObj := (Smalltalk classNamed: current) ifNotNilDo: [:cls | cls new].
			]
		].! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: 'sm 5/25/2011 17:26'!
determineSuiteOrBenchmark
	self parseBenchmarkOrSuite: current.! !

!SMarkHarnessArgumentParser methodsFor: 'accessing' stamp: ''!
harness: aHarness
	harness := aHarness! !

!SMarkHarnessArgumentParser methodsFor: 'helper' stamp: 'sm 5/25/2011 17:29'!
instructRunner
	suite := suiteClass new.
	specificBenchmark ifNotNil: [
		suite runOnly: specificBenchmark.
	].
	
	runner suite: suite.
 	runner iterations: iterations.
	runner processes: processes.
	runner problemSize: problemSize.! !

!SMarkHarnessArgumentParser methodsFor: 'parsing' stamp: 'sm 5/25/2011 17:27'!
parse: argumentsArray
	arguments := argumentsArray.
	numParams := arguments size.
	
	i := 2.
	current := arguments at: i.
	currentObj := (Smalltalk classNamed: current) ifNotNilDo: [:cls | cls new].
		
	self determineRunner.
	self determineReporter.
	
	self determineSuiteOrBenchmark.
	
	self determineBenchmarkParameters.
	
	self instructRunner.
	
	^ {runner. reporter}! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
parseBenchmarkOrSuite: aBenchmarkOrSuite
	"Identify the benchmark suite or suite and benchmark method
	 that should be executed. The string should be of the format 'Class>>benchName' or 'Class.benchName' for shell/bash compatibility.
	 Accepts a string, class, or array.
	 Returns, a class, or an array of a class and a symbol."
	| parsed |
	(aBenchmarkOrSuite isKindOf: Class)
		ifTrue: [
			suiteClass := aBenchmarkOrSuite.
			^ suiteClass
		].
	  
	(aBenchmarkOrSuite isKindOf: Array)
		ifTrue:  [ parsed := aBenchmarkOrSuite. ]
		ifFalse: [ parsed := aBenchmarkOrSuite findTokens: '>.'. ].
	
	((parsed size > 2) or: [parsed size < 1])
				ifTrue: [ Error signal: 'The passed argument has to represent two elements. A class/classname and a method symbol' ].
	
	suiteClass := parsed first.
	
	(suiteClass isKindOf: Class) 
		ifFalse: [ suiteClass := Smalltalk at: (suiteClass asSymbol) ifAbsent: [Error signal: 'Class that was supposed to represent a benchmark suite was not found: ', suiteClass asString ]].
	
	parsed size = 1
		ifTrue: [^suiteClass].

	specificBenchmark := parsed second asSymbol.
	
	^ { suiteClass. specificBenchmark }
! !

Object subclass: #SMarkReporter
	instanceVariableNames: 'runner stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkReporter commentStamp: 'StefanMarr 3/20/2011 19:55' prior: 0!
PBenchmarkReporter has not been documented yet. The class comment should describe the purpose of the class, its collaborations and its variables.

Instance Variables:
	runner	<PBenchmarkRunner>
	stream	<NSDecoderDeflateStream | SocketStream | ThreadSafeTranscript | WriteStream>
		
Example:

	f := TextStream on: String new.
	PBenchmarkSimpleStatisticsReporter reportFor: (PTestBenchmarkRunnerSuiteForAutosizing run: 10) on: f.
	f contents!

!SMarkReporter class methodsFor: 'defaults' stamp: ''!
defaultOutputDestination
	^ ScriptConsole! !

!SMarkReporter class methodsFor: 'defaults' stamp: ''!
defaultReporter
	^ SMarkSimpleStatisticsReporter! !

!SMarkReporter class methodsFor: 'reporting' stamp: ''!
reportFor: aRunner
	self reportFor: aRunner on: self defaultOutputDestination.! !

!SMarkReporter class methodsFor: 'reporting' stamp: ''!
reportFor: aRunner on: aStream
	| reporter |
	reporter := self new.
	reporter runner: aRunner.
	reporter outputStream: aStream.
	reporter report.
	^ reporter.! !

!SMarkReporter methodsFor: 'reporting' stamp: ''!
benchmarkFooter: aName
	stream cr.! !

!SMarkReporter methodsFor: 'reporting' stamp: ''!
benchmarkHeader: aName
	stream << 'Benchmark ' << (aName asString); cr.! !

!SMarkReporter methodsFor: 'reporting' stamp: ''!
footer
	"No output at the moment"
	^ self! !

!SMarkReporter methodsFor: 'reporting' stamp: ''!
header
	| suiteName |
	suiteName := runner suite class name asString.
	stream << 'Report for: ' << suiteName; cr.! !

!SMarkReporter methodsFor: 'initialization' stamp: ''!
initialize
	stream := self class defaultOutputDestination.! !

!SMarkReporter methodsFor: 'accessing' stamp: ''!
outputStream: aStream
	stream := aStream  ! !

!SMarkReporter methodsFor: 'accessing' stamp: ''!
report
	self header.
	
	runner results keysAndValuesDo: [:key :value |
		self benchmarkHeader: key.
		self reportAllRuns: value of: key.
		self benchmarkFooter: key.
	].

	self footer.
	^ self! !

!SMarkReporter methodsFor: 'reporting' stamp: 'CamilloBruni 9/5/2011 13:13'!
reportAllRuns: aListOfResults of: benchmark
	aListOfResults do: [:result |
		result criteria keysAndValuesDo: [:benchName :timer |
			stream << benchName << ': ' << (timer totalTime asString, 'ms'); cr.]]! !

!SMarkReporter methodsFor: 'accessing' stamp: ''!
runner: aRunner
	runner := aRunner.! !

SMarkReporter subclass: #SMarkSimpleStatisticsReporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

SMarkSimpleStatisticsReporter subclass: #ReBenchReporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!ReBenchReporter methodsFor: 'as yet unclassified' stamp: 'StefanMarr 5/16/2011 15:23'!
benchmarkHeader: aName
	^ self! !

!SMarkSimpleStatisticsReporter methodsFor: 'statistics' stamp: 'StefanMarr 3/20/2011 12:05'!
confidenceVariance: times
	| numMeasurements |
	numMeasurements := times size.
	(numMeasurements >= 30) 
		ifTrue: [
			^ (self gaussianConfidenceFactor) * (times stdev) / (numMeasurements asFloat sqrt)].
		
	"use the students T distribution for small probe counts"
	^ (self studentsTConfidenceFactorFor: numMeasurements) * (times stdev) / (numMeasurements asFloat sqrt)! !

!SMarkSimpleStatisticsReporter methodsFor: 'statistics' stamp: 'StefanMarr 3/20/2011 11:49'!
gaussianConfidenceFactor
	"used for large probe counts >= 30"
	"1 ~ 68.27%"
	"1.644853626951 ~ 90%"
	"2 ~ 95.45%"
	^ 1.644853626951! !

!SMarkSimpleStatisticsReporter methodsFor: 'reporting' stamp: 'StefanMarr 5/16/2011 16:08'!
reportAllRuns: aListOfResults of: benchmark
	| criteria |

	criteria := aListOfResults first criteria.
	
	criteria keysDo: [:criterion |
		| times |
		times := self resultsFor: criterion from: aListOfResults.
		self reportResult: times for: criterion of: benchmark.
		stream cr.
	].! !

!SMarkSimpleStatisticsReporter methodsFor: 'reporting' stamp: 'CamilloBruni 9/6/2011 10:41'!
reportResult: aResultsArray for: aCriterion of: benchmark
	| convidenceVariance significantDigits |

	stream << benchmark <<  ' ' <<  aCriterion <<  ': iterations='.
	aResultsArray size printOn: stream .
	stream << ' runtime: '.
	
	aResultsArray size < 2 ifTrue: [
		aResultsArray average printOn: stream.
		stream << 'ms'.
		^ self.
	].
	
	convidenceVariance := self confidenceVariance: aResultsArray.  
	
	"only print significant "
	significantDigits := self significantDigits: convidenceVariance.

	aResultsArray average printOn: stream showingDecimalPlaces: significantDigits.
	stream << 'ms +/-'.
	convidenceVariance printOn: stream showingDecimalPlaces: significantDigits.! !

!SMarkSimpleStatisticsReporter methodsFor: 'helper' stamp: 'StefanMarr 5/16/2011 16:02'!
resultsFor: aCriterion from: aListOfResults
	^aListOfResults collect: [:result | (result criteria at: aCriterion) totalTime]
	! !

!SMarkSimpleStatisticsReporter methodsFor: 'statistics' stamp: 'StefanMarr 3/20/2011 11:46'!
significantDigits: confidenceVariance
	confidenceVariance = 0 
		ifTrue: [ ^ 2].
	
	confidenceVariance >= 10
		ifTrue: [ ^ 0].
	
	^ 1 - (confidenceVariance log floor)! !

!SMarkSimpleStatisticsReporter methodsFor: 'statistics' stamp: 'StefanMarr 3/20/2011 12:03'!
studentsTConfidenceFactorFor: aNumberOfMeasurements
	"used for small probe counts < 30"
	"the students T distribution sucks to calculate since the value depends on the probeCout"
	"these values are for a confidence interval of ~90%"
	| values |
	values := Array new: 30.
	values at: 1  put: 6.314.
	values at: 2  put: 2.920.
	values at: 3  put: 2.353.
	values at: 4  put: 2.132.
	values at: 5  put: 2.015.
	values at: 6  put: 1.943.
	values at: 7  put: 1.895.
	values at: 8  put: 1.860.
	values at: 9  put: 1.833.
	values at: 10 put: 1.812.
	values at: 11 put: 1.796.
	values at: 12 put: 1.782.
	values at: 13 put: 1.771.
	values at: 14 put: 1.761.
	values at: 15 put: 1.753.
	values at: 16 put: 1.746.
	values at: 17 put: 1.740.
	values at: 18 put: 1.734.
	values at: 19 put: 1.729.
	values at: 20 put: 1.725.
	values at: 21 put: 1.721.
	values at: 22 put: 1.717.
	values at: 23 put: 1.714.
	values at: 24 put: 1.711.
	values at: 25 put: 1.708.
	values at: 26 put: 1.706.
	values at: 27 put: 1.703.
	values at: 28 put: 1.701.
	values at: 29 put: 1.699.
	values at: 30 put: 1.697.
	^ values at: aNumberOfMeasurements.
	! !

!SMarkSimpleStatisticsReporter methodsFor: 'helper' stamp: 'StefanMarr 3/20/2011 19:40'!
totalResultsFor: aListOfResults
	^aListOfResults collect: [:timer | timer total]
	! !

Object subclass: #SMarkResult
	instanceVariableNames: 'time benchName suite criteria'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkResult commentStamp: 'StefanMarr 3/18/2011 23:45' prior: 0!
A benchmark result is characterized by:
 - the total execution time (#total is the least a benchmark results in)
 - the name of the benchmark that was executed
 - the suite object specifies the used input used for the benchmark
 - dictionary of additional the criteria and the related timings

A benchmark can produced multiple resuts for different criteria. The standard criterion is #total.!

!SMarkResult methodsFor: 'accessing' stamp: ''!
benchmarkName
	^ benchName! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
benchmarkName: aString
	benchName := aString! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
criteria
	^ criteria! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
criteria: aCollectionOfTimers
	criteria := aCollectionOfTimers! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
suite
	^ suite! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
suite: aBenchmarkSuite
	suite := aBenchmarkSuite! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
total
	^ time! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
total: aTime
	time := aTime! !

Object subclass: #SMarkRunner
	instanceVariableNames: 'numIterations suite runner results currentBenchmark timers problemSize numProcesses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

SMarkRunner subclass: #SMarkAutosizeRunner
	instanceVariableNames: 'targetTime innerLoopIterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkAutosizeRunner class methodsFor: 'defaults' stamp: 'StefanMarr 3/19/2011 19:17'!
defaultTargetTime
	"300 milliseconds seems to be a reasonable target time for most problems.
	 It is a compromise between the general measurment noise as well as timer accuracy
	 and the absolute runtime of benchmarks"
	^ 300! !

!SMarkAutosizeRunner methodsFor: 'initialization' stamp: 'StefanMarr 3/19/2011 19:59'!
initialize
	super initialize.
	targetTime := self class defaultTargetTime.! !

!SMarkAutosizeRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 3/20/2011 10:27'!
performBenchmark: aSelector
	"First determine a useful number of inner loop iterations until the targetTime is reached."
	| execTime i |
	"make sure no timers are recorded for this"
	timers := nil.
	
	i := 1.
	execTime := Time millisecondsToRun: [ suite perform: aSelector. ].
	
	[ execTime > targetTime ] whileFalse: [
		i := i * 2. "Was thinking of doing something fancy here, but just go with this simple staight-forward solution"
		execTime := Time millisecondsToRun: [ 1 to: i do: [:ignored| suite perform: aSelector]].
	].

	innerLoopIterations := i.
	
	"Then start executing the benchmark"
	^ super performBenchmark: aSelector.! !

!SMarkAutosizeRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 3/20/2011 10:49'!
runBaseBenchmark
	"baseBenchmark is not supported with autosizing. I do not see how that can be made possible since all different benchmarks will have different number of iterations, and the only way how a consistent baseline could be found would be to normalize the results, but well, incooprorating the baseline measurement with the statistical evaluation is harder than just substracting a one time value..., I am not going to do that here for the moment. Stefan 2011-03-20"
	
	(suite respondsTo: #baseBenchmark) 
		ifFalse: [ ^ nil ].
	
	"I decided to go here with a silent solution to avoid thinking about logging frameworks and Transcript to console convertion..."
	self recordResults: (self class defaultTimer new: 'total') for: #baseBenchmark  ! !

!SMarkAutosizeRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/19/2011 19:18'!
targetTime
	"Target time in milliseconds"
	^ targetTime! !

!SMarkAutosizeRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/19/2011 19:18'!
targetTime: anIntInMilliseconds
	"Target time in milliseconds"
	targetTime := anIntInMilliseconds! !

!SMarkAutosizeRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 3/20/2011 10:31'!
timedBenchmarkExecution: aSelector
	"Will do a timed execution of the benchmark and return the result timer"
	| timer |
	timers := Dictionary new.

	timer := self createTimer: 'total'.
	
	timer start.
	1 to: innerLoopIterations do: [:ignored|
		suite perform: aSelector.
	].
	timer stop.
	
	self recordResults: timer for: aSelector.
	
	^ timer! !

SMarkAutosizeRunner subclass: #SMarkTestAutosizeRunner
	instanceVariableNames: 'performedBenchmarks notifyRunnerWasExecuted setUpCount tearDownCount didTearDownBenchC didSetUpBenchB baseBenchmarkExecuted'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
baseBenchmarkExecuted
	^ baseBenchmarkExecuted! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
countSetUpInvoke
	setUpCount := setUpCount + 1! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
countTearDownInvoke
	tearDownCount := tearDownCount + 1! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
countWasExecuted
	^ notifyRunnerWasExecuted ! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
didPerform: aSymbol
	performedBenchmarks add: aSymbol.! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
didRunBaseBenchmark
	baseBenchmarkExecuted := true.! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
didSetUpBenchB
	didSetUpBenchB := true.! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
didTearDownBenchC
	didTearDownBenchC := true.! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 23:24'!
initialize
	super initialize.

	setUpCount := 0.
	tearDownCount := 0.
	notifyRunnerWasExecuted := 0.
	
	baseBenchmarkExecuted := false.

	performedBenchmarks := Set new.
	
	"set the targetTime to something which is more reasonable for testing"
	targetTime := 50.! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
isNotifyRunnerWasExecutedSet
	^ notifyRunnerWasExecuted! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
notifyRunnerWasExecuted
	notifyRunnerWasExecuted := notifyRunnerWasExecuted + 1! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
performedBenchmarks
	^ performedBenchmarks! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
setUp
	notifyRunnerWasExecuted := false! !

!SMarkRunner class methodsFor: 'defaults' stamp: 'StefanMarr 3/21/2011 11:43'!
defaultNumberOfIterations
	^ 1! !

!SMarkRunner class methodsFor: 'defaults' stamp: 'StefanMarr 5/14/2011 11:09'!
defaultNumberOfProcesses
	^ 1! !

!SMarkRunner class methodsFor: 'defaults' stamp: 'StefanMarr 3/20/2011 20:56'!
defaultTimer
	^ SMarkTimer! !

!SMarkRunner class methodsFor: 'benchmarking' stamp: 'StefanMarr 3/20/2011 19:51'!
execute: aSuite
	^ self execute: aSuite with: 1.! !

!SMarkRunner class methodsFor: 'benchmarking' stamp: 'StefanMarr 3/20/2011 19:51'!
execute: aSuite with: nIterations
	| runner |
	runner := self new.
	aSuite runner: runner.
	runner suite: aSuite.
	runner iterations: nIterations.  
	runner execute.
	^ runner! !

!SMarkRunner methodsFor: 'helper' stamp: 'StefanMarr 3/20/2011 10:28'!
createTimer: name
	"Create and register a new timer for the current benchmark"
	| timer |
	timer := self class defaultTimer new: name.
	
	timers ifNotNil: [
		timers at: name put: timer.
	].
	
	^ timer.! !

!SMarkRunner methodsFor: 'execution' stamp: 'StefanMarr 3/13/2011 23:05'!
execute
	suite run.
	self runBaseBenchmark.
	^ results
	! !

!SMarkRunner methodsFor: 'initialization' stamp: 'StefanMarr 5/14/2011 11:10'!
initialize
	super initialize.
	numIterations := self class defaultNumberOfIterations.
	numProcesses  := self class defaultNumberOfProcesses.
	results := Dictionary new.! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/21/2011 12:20'!
iterations
	^ numIterations! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/21/2011 12:20'!
iterations: anInteger
	numIterations := anInteger! !

!SMarkRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 11:50'!
performBenchmark: aSelector
	currentBenchmark := aSelector.
	
	1 to: numIterations do: [:i|
		"self timedBenchmarkExecution: aSelector."
		suite runBenchmark: aSelector.  
	].	
	
	currentBenchmark := nil.
	
	^ results at: (suite benchmarkNameForSelector: aSelector)! !

!SMarkRunner methodsFor: 'printing' stamp: 'CamilloBruni 9/5/2011 13:13'!
printOn: aStream
	^ self reportOn: aStream.! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 4/15/2011 16:08'!
problemSize
	^ problemSize! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 4/15/2011 16:09'!
problemSize: aValue
	problemSize := aValue! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 5/14/2011 11:08'!
processes
	"The standard runner does use only a single process, but in case a benchmark supports parallelism it can query for the intended degree of parallelism"
	^ numProcesses! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 5/14/2011 11:08'!
processes: anInt
	"The standard runner does use only a single process, but a benchmark can use that to do its own parallelism"
	numProcesses := anInt! !

!SMarkRunner methodsFor: 'helper' stamp: 'StefanMarr 3/20/2011 21:01'!
recordResults: timer for: aSelector
	| result name |
	name := suite benchmarkNameForSelector: aSelector.
	
	result := SMarkResult new.
	result total: timer totalTime.
	result benchmarkName: name.
	result suite: suite.
	result criteria: timers.
	
	(results at: name ifAbsentPut: OrderedCollection new) add: result.! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 4/15/2011 16:46'!
report
	SMarkReporter defaultReporter reportFor: self.  
	! !

!SMarkRunner methodsFor: 'reporting' stamp: 'sm 5/25/2011 17:50'!
reportConfiguration: aStream
	aStream << 'Runner Configuration:';cr.
	aStream << ('  iterations: ', numIterations asString); cr.
	aStream << ('  processes: ', numProcesses asString); cr.
	aStream << ('  problem size: ', problemSize asString); cr.
! !

!SMarkRunner methodsFor: 'reporting' stamp: 'StefanMarr 4/15/2011 16:47'!
reportOn: aStream
	SMarkReporter defaultReporter reportFor: self on: aStream  
	! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/18/2011 19:41'!
results
	^ results! !

!SMarkRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 3/18/2011 18:42'!
runBaseBenchmark
	"In certain sitatuations it is one wants a baseline that is incooprated in all 
	 benchmark results to be substracted from the final values.
	
	#baseBenchmark can be used to charaterize such a baseline"
	
	(suite respondsTo: #baseBenchmark) 
		ifFalse: [ ^ nil ].
	
	^ self performBenchmark: #baseBenchmark.! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/20/2011 18:45'!
suite
	^ suite! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 4/15/2011 16:14'!
suite: aBenchmarkSuite
	suite := aBenchmarkSuite.
	suite runner: self.! !

!SMarkRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 3/19/2011 22:08'!
timedBenchmarkExecution: aSelector
	"Will do a timed execution of the benchmark and return the result timer"
	| timer |
	timers := Dictionary new.

	timer := self createTimer: 'total'.
	
	timer start.
	suite perform: aSelector.
	timer stop.
	
	self recordResults: timer for: aSelector.
	
	^ timer! !

SMarkRunner subclass: #SMarkTestRunner
	instanceVariableNames: 'performedBenchmarks notifyRunnerWasExecuted setUpCount tearDownCount didTearDownBenchC didSetUpBenchB baseBenchmarkExecuted'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 18:35'!
baseBenchmarkExecuted
	^ baseBenchmarkExecuted! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:36'!
countSetUpInvoke
	setUpCount := setUpCount + 1! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:39'!
countTearDownInvoke
	tearDownCount := tearDownCount + 1! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:44'!
countWasExecuted
	^ notifyRunnerWasExecuted ! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:00'!
didPerform: aSymbol
	performedBenchmarks add: aSymbol.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:56'!
didRunBaseBenchmark
	baseBenchmarkExecuted := true.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:43'!
didSetUpBenchB
	didSetUpBenchB := true.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:42'!
didTearDownBenchC
	didTearDownBenchC := true.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 12:14'!
hasSetUpBenchB
	^ didSetUpBenchB.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 12:21'!
hasTearDownBenchC
	^ didTearDownBenchC! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:49'!
initialize
	super initialize.

	setUpCount := 0.
	tearDownCount := 0.
	notifyRunnerWasExecuted := 0.
	
	baseBenchmarkExecuted := false.

	performedBenchmarks := OrderedCollection new.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/15/2011 22:16'!
isNotifyRunnerWasExecutedSet
	^ notifyRunnerWasExecuted! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:44'!
notifyRunnerWasExecuted
	notifyRunnerWasExecuted := notifyRunnerWasExecuted + 1! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:00'!
performedBenchmarks
	^ performedBenchmarks! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/15/2011 22:15'!
setUp
	notifyRunnerWasExecuted := false! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 12:27'!
setUpCount
	^ setUpCount! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 12:39'!
tearDownCount
	^ tearDownCount! !

SMarkRunner subclass: #SMarkWeakScalingRunner
	instanceVariableNames: 'numInnerIterations runningProcesses completionSignal runningProcessesMtx'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

SMarkWeakScalingRunner subclass: #SMarkTestWeakScalingRunner
	instanceVariableNames: 'setUpCount tearDownCount notifyRunnerWasExecuted performedBenchmarks baseBenchmarkExecuted didSetUpBenchB didTearDownBenchC'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:44'!
baseBenchmarkExecuted
	^ baseBenchmarkExecuted! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:41'!
countSetUpInvoke
	setUpCount := setUpCount + 1! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:41'!
countTearDownInvoke
	tearDownCount := tearDownCount + 1! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:41'!
countWasExecuted
	^ notifyRunnerWasExecuted ! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:41'!
didPerform: aSymbol
	performedBenchmarks add: aSymbol.! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
didRunBaseBenchmark
	baseBenchmarkExecuted := true.! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
didSetUpBenchB
	didSetUpBenchB := true.! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
didTearDownBenchC
	didTearDownBenchC := true.! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
initialize
	super initialize.

	setUpCount := 0.
	tearDownCount := 0.
	notifyRunnerWasExecuted := 0.
	
	baseBenchmarkExecuted := false.

	performedBenchmarks := OrderedCollection new.! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 13:20'!
isNotifyRunnerWasExecutedSet
	^ notifyRunnerWasExecuted > 0! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
notifyRunnerWasExecuted
	notifyRunnerWasExecuted := notifyRunnerWasExecuted + 1! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
performedBenchmarks
	^ performedBenchmarks! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:43'!
setUp
	notifyRunnerWasExecuted := false! !

!SMarkWeakScalingRunner class methodsFor: 'defaults' stamp: 'StefanMarr 4/15/2011 15:42'!
defaultNumberOfInnerIterations
	"The number of iterations of the inner loop
	 in which the benchmark is executed."
	^ 1! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'sm 5/30/2011 17:13'!
execute: aSelector withProcesses: numberOfProcesses withTimer: timer
	"This case is meant for all cases. REM: this is also used for numProc==1 to be able to measure the process start overhead in all cases.
	 It will start the processes and wait for their completion."
	
	| processes |
	processes			:= Array new: numberOfProcesses.
	runningProcessesMtx := Semaphore forMutualExclusion.
	completionSignal		:= Semaphore new.
	runningProcesses := numberOfProcesses.
	
	"First initialize the processes"
	1 to: numberOfProcesses do: [ :procNum |
		| proc |
		proc := SMarkWeakScalingRunnerExecutor createFor: aSelector for: numInnerIterations with: self and: suite.
		proc priority: Processor highestPriority.
		proc name: (self class name, '-',  procNum asString).
		processes at: procNum put: proc.
		"On: procNum"
	].
	
	"Now, execute the benchmark and do the timing now"
	timer start.
	1 to: numberOfProcesses do: [ :procNum |
		(processes at: procNum) resume.
	].
	completionSignal wait.
	timer stop.
	! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'sm 5/30/2011 16:17'!
executorCompleted
	runningProcessesMtx critical: [
		runningProcesses := runningProcesses - 1.
		(runningProcesses == 0) ifTrue: [
			completionSignal signal.
		]
	]! !

!SMarkWeakScalingRunner methodsFor: 'initialization' stamp: 'StefanMarr 4/15/2011 15:43'!
initialize
	super initialize.
	numProcesses			:= self class defaultNumberOfProcesses.
	numInnerIterations	:= self class defaultNumberOfInnerIterations.
! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 15:43'!
innerIterations
	"The number of inner iterations the benchmark is executed inside a processes"
	^ numInnerIterations! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 15:43'!
innerIterations: anInteger
	"The number of inner iterations the benchmark is executed inside a processes"
	numInnerIterations := anInteger! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 10:54'!
processes
	^ numProcesses! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 10:53'!
processes: anInteger
	numProcesses := anInteger! !

!SMarkWeakScalingRunner methodsFor: 'reporting' stamp: 'sm 5/25/2011 17:50'!
reportConfiguration: aStream
	super reportConfiguration: aStream.
	aStream << ('inner iterations: ', numInnerIterations asString); cr.! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'sm 5/25/2011 18:31'!
timedBenchmarkExecution: aSelector
	"Will do a timed execution of the benchmark and return the result timer"
	| timer |
	timers := Dictionary new.

	timer := self createTimer: 'total'.
	
	self execute: aSelector withProcesses: numProcesses withTimer: timer.
	
	self recordResults: timer for: aSelector.
	
	^ timer! !

Object subclass: #SMarkSuite
	instanceVariableNames: 'runner selectedBenchmarks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkSuite commentStamp: 'StefanMarr 3/20/2011 20:57' prior: 0!
A Benchmark Suite is a set of benchmarks and it knows what exactly needs to be executed.
However, it does not really know how to execute it.
It knows all the magic, that is, how to set up and tear down the environment for the benchmarks, but does not have the knowledge of how many iterations need to be done and how to evaluate any results that might be produced.!

SMarkSuite subclass: #SMarkCompiler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Classic'!

!SMarkCompiler class methodsFor: 'script entry' stamp: ''!
defaultNumberOfIterations
	^ 50! !

!SMarkCompiler class methodsFor: 'script entry' stamp: ''!
defaultNumberOfProcesses
	^ 8! !

!SMarkCompiler methodsFor: 'benchmarking' stamp: ''!
benchCompiler
	"was Benchmark>>testCompiler"
	SMarkCompilerTargetClass new compile:
		'somethingDummy: aBlock repeated: nTimes | i emptyBlock emptyTime blockTime |
		nTimes > 1000 ifTrue: [^self time: aBlock repeatedK: nTimes // 1000].
		emptyBlock := [].
		emptyTime := Time millisecondsToRun:
						[i := 0.
						 [(i := i + 1) <= nTimes] whileTrue: [emptyBlock value]].
		blockTime := Time millisecondsToRun:
						[i := 0.
						 [(i := i + 1) <= nTimes] whileTrue: [aBlock value]].
		^blockTime - emptyTime'.! !

SMarkSuite subclass: #SMarkLoops
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Classic'!

!SMarkLoops methodsFor: 'benchmarking' stamp: ''!
benchFloatLoop
	| a |
	a := 1.23.
	[a < 50000] whileTrue: [
		a := a + 0.6.
		a := a + 0.4.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: ''!
benchIntLoop
	| i |
	i := 50000.
	[i > 0] whileTrue: [
		i := i - 1.
	].! !

SMarkSuite subclass: #SMarkSlopstone
	instanceVariableNames: 'testParams testBlocks output'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Classic'!

!SMarkSlopstone commentStamp: 'nishis 2/21/98 06:26' prior: 0!
modified for Squeak by nishis@urban.ne.jp
I added one class methods.
and modified execute method.  block variables are renamed for Squeak.
Thanx to Mr. Tim Olson, Mr. Kohler Markus, Mr. Tim Rowledge, Mr. John Maloney, Mr. Ian Piumarta.

original comments
	NAME			STones80
	AUTHOR			bruce@utafll.uta.edu (Bruce Samuelson)
	FUNCTION		low and medium level benchmarks for ST80 and ST/V
	ST-VERSIONS		pre R4.0, R4.0, R4.1, ST/V
	PREREQUISITES	need floating point hardware or emulation
	CONFLICTS		none
	DISTRIBUTION	world
	VERSION			1.0
	DATE			April 16, 1993

SUMMARY The filein includes two classes: Slopstones (Smalltalk Low level
OPeration Stones) and Smopstones (Smalltalk Medium level OPeration Stones).
Each includes seven cpu intensive benchmarks. They work equally well with
16-bit and 32-bit implementations and are designed to be portable to all
Smalltalk versions from ParcPlace and Digitalk. They are normalized to the
author's 486/33 Windows 3.1 machine running ParcPlace VisualWorks 1.0.
Results have been posted to the Usenet group comp.lang.smalltalk and form the
basis of an article that is scheduled to be published in the June issue of The
Smalltalk Report.

The only difference between the ST80 (STones80) and ST/V (STonesV) filein is
in the messages that define the classes in the first few lines of code. The
ST80 messages specify the class category and message protocol, which are not
used in ST/V.

Bruce Samuelson

!

!SMarkSlopstone methodsFor: 'benchmarking' stamp: ''!
benchStone
	"modified by nishis"

	| n nTests iters times speeds stones scale printA printB printC param count speed0 expln block time iter speed stone geoMean gm power |
	n := self problemSize.	"Number of times each test block will be evaluated."
	n > SmallInteger maxVal
		ifTrue: [ self error: 'Count exceeded max small int.' ].
	
	nTests := testParams size.
	nTests = testBlocks size
		ifFalse: [ self error: 'Inconsistent test count.' ].
	iters := OrderedCollection new.
	times := OrderedCollection new.
	speeds := OrderedCollection new.
	stones := OrderedCollection new.
	scale := 1000.	"So iterations can be reported as 1000s of iterations"	"The following blocks are restricted to two args by ST/V-DOS."
	printA := [ :t1 :t2 | 
	"[:iter1 :time1 |"
	output cr.
	output nextPutAll: (t1 / scale) rounded printString.
	output nextPutAll: '     '.
	output nextPutAll: t2 printString.
	output nextPutAll: '     ' ].
	printB := [ :t1 :t2 | 
	"[:speed1 :slop1 |"
	output nextPutAll: (t1 / scale) rounded printString.
	output nextPutAll: '     '.
	output nextPutAll: t2 printString.
	output nextPutAll: '     ' ].
	printC := [ :t1 | 
	":expln1 |"
	output nextPutAll: t1 ].
	output
		nextPutAll:
			'

1000s    time    1000s of
itera-   sec-    iterations   slop-
tions    onds    per sec      stones   explanation
'.
	1 to: nTests do: [ :i | 
		param := testParams at: i.
		count := param at: 1.	"repetitions of a test inside its block"
		speed0 := (param at: 2) * scale.	"iters/sec for a one-slopstone machine"
		expln := param at: 3.
		block := testBlocks at: i.
		time := Time millisecondsToRun: [ n timesRepeat: block ].
		time := (time max: 1) / 1000.0.	"time is now in seconds"
		iter := count * n.
		speed := iter / time.
		stone := speed / speed0.
		iters add: iter.
		times add: time.
		speeds add: speed.
		stones add: stone.
		printA value: iter value: time.
		printB value: speed value: stone.
		printC value: expln ].
	geoMean := [ :t1 | 
	"[:numbers |"
	gm := 1.
	power := 1 / nTests.	"t2 as number"
	t1 do: [ :t2 | gm := gm * (t2 raisedTo: power) ].
	gm ].
	output cr.
	printA value: (geoMean value: iters) value: (geoMean value: times).
	printB value: (geoMean value: speeds) value: (geoMean value: stones).
	printC value: 'geometric mean'.
	output
		cr;
		cr;
		nextPutAll: 'Benchmarks complete.';
		cr! !

!SMarkSlopstone methodsFor: 'defaults' stamp: ''!
defaultProblemSize
	^ 1! !

!SMarkSlopstone methodsFor: 'benchmarking' stamp: ''!
problemSize
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

!SMarkSlopstone methodsFor: 'benchmarking' stamp: 'nishis 2/14/98 02:33'!
readme

	"INTRODUCTION 
	 
	Slopstone: Smalltalk Low level OPeration Stones 
	Portable Low Level Benchmarks for ST80 and ST/V (using 16-bit 
	SmallIntegers) Placed in public domain January 1993  (c) Bruce 
	Samuelson Permission is given to place this in public Smalltalk archives 
	 
	Use monospaced fonts if possible to view the methods in this class. 
	 
	(1) Collect garbage if supported (2) do 'SlopstoneBenchmark new 
	runBenchmark'. Results are printed in the Transcript window. 
	Post results for your machines to comp.lang.smalltalk or 
	mail them to bruce@ling.uta.edu or bruce@utafll.uta.edu. 
	 
	DISCUSSION 
	    
	This readme method would normally be in the class comment for ST80. 
	ST/V-DOS doesn't support class comments. 
	  
	The benchmarks test strictly low level operations. They do not test 
	higher level operations such as forming sets, sorting, or streaming, nor 
	do they test 
	applications. They also do not test user interface operations because of 
	the non-portability of this area of Smalltalk and its sensitivity to the 
	performance of the video subsystem. The tests are cpu bound. They do 
	not access files and should not cause disk paging. 
	 
	The benchmarks use loop counts of 16000 because SmallIntegers cannot 
	exceed 16383 for ST/V-DOS. 16-bit implementions would perform worse 
	with large loop 
	counts. The benchmarks are also suitable for testing 32-bit versions of 
	Smalltalk. 
	 
	DEFINITION OF REFERENCE MACHINE (ONE SLOPSTONE) 
	 
	The following machine is the one on which I developed these 
	benchmarks. By 
	convention it is defined to operate at one slopstone. It's a mid range 
	performer for current ParcPlace versions of Smalltalk. 
	 
	Hardware: Amax 486DX/33 (includes internal floating point processor 
	and internal 8K cache), 256K external cache, 16MB RAM. 
	 
	Software: ParcPlace VisualWorks 1.0, Windows 3.1, DOS 5.0 (plain vanilla 
	setup). 
	 
	COMPARISON TO XEROX DORADO 
	 
	For reference, the machine runs at 649% of a Dorado on ParcPlace 
	benchmarks for ST80 4.1. Its fast video card helps on these PPS 
	benchmarks. I didn't run 
	them for VisualWorks 1.0. It would be somewhat slower because there 
	are vastly 
	more classes. 
	 
	EXAMPLE RESULTS FOR REFERENCE MACHINE 
	 
	1000s    time    1000s of 
	itera-   sec-    iterations   slop- 
	tions    onds    per sec      stones   explanation 
	 
	3808     0.577   6600         1.0      add integers 
	 544     2.262    240         1.0      add floats 
	 960     1.088    882         1.0      access strings 
	 320     0.908    352         1.0      create objects 
	 160     1.49     107         1.0      copy objects 
	 480     1.129    425         1.0      perform selectors 
	 896     1.237    724         1.0      evaluate blocks 
	 
	 640     1.151    555         1.0      geometric mean"
	^ self! !

!SMarkSlopstone methodsFor: 'benchmarking' stamp: ''!
setUp
 
	"Numbers in testParams represent the following:
 
	Column 1   number of internal repetitions for each test inside its block
	Column 2   thousands of iterations per second for a one-slopstone machine."
 
	output := TextStream with: String new.

	testParams := OrderedCollection new.
 
	testParams
		add: #(238 6600 'add integers');
		add: #( 34  240 'add floats');
		add: #( 60  882 'access strings');
		add: #( 20  352 'create objects');
		add: #( 10  107 'copy objects');
		add: #( 30  425 'perform selectors');
		add: #( 56  724 'evaluate blocks').
 
	testBlocks := OrderedCollection new.
 
	"In the integer addition test, I originally had 340 internal repetitions.
	This caused the Digitalk compiler to blow up. The ParcPlace compiler compiled
	it ok but interestingly addition performed at slightly more than 50% of the
	performance with an internal count of 238. Perhaps something magical happens
	at 256."

	testBlocks
		add: [1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
			1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
			1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
			1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
			1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
			1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
			1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1];
 
		add: [1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+
			1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0];
 
		add: ['a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
			'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
			'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
			'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
			'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
			'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
			'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
			'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
			'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
			'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1];
 
		add: [Object new. Object new. Object new. Object new. Object new.
			Object new. Object new. Object new. Object new. Object new.
			Object new. Object new. Object new. Object new. Object new.
			Object new. Object new. Object new. Object new. Object new];
 
		add: [Object new copy copy copy copy copy copy copy copy copy copy];
 
		add: [0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself.
			0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself.
			0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself.
			0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself.
			0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself.
			0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself.
			0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself.
			0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself.
			0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself.
			0 perform: #yourself. 0 perform: #yourself. 0 perform: #yourself];
 
		add: [[] value. [] value. [] value. [] value. [] value. [] value. [] value.
			[] value. [] value. [] value. [] value. [] value. [] value. [] value.
			[] value. [] value. [] value. [] value. [] value. [] value. [] value.
			[] value. [] value. [] value. [] value. [] value. [] value. [] value.
			[] value. [] value. [] value. [] value. [] value. [] value. [] value.
			[] value. [] value. [] value. [] value. [] value. [] value. [] value.
			[] value. [] value. [] value. [] value. [] value. [] value. [] value.
			[] value. [] value. [] value. [] value. [] value. [] value. [] value]! !

SMarkSuite subclass: #SMarkSmopstone
	instanceVariableNames: 'testParams testBlocks output'
	classVariableNames: 'IsSupportRecursiveBlock WhichFloatPrintOn'
	poolDictionaries: ''
	category: 'SMark-Classic'!

!SMarkSmopstone commentStamp: 'nishis 2/21/98 06:26' prior: 0!
modified for Squeak by nishis@urban.ne.jp
see the comment of SlopstoneBenchmark class.
I added all class methods and initialization>initilize method,
modified:
	streamTestsOn: method.  In some case use testPrintOn: s base: instead of printOn:,
	primesUpTo: method. From recursively calling a block to whileTrue: block.
	sorcerersApprentice method. From recursively calling a block to  recursive method.
	execute and setup method.
Thanx to Mr. Tim Olson, Mr. Kohler Markus, Mr. Tim Rowledge, Mr. John Maloney, Mr. Ian Piumarta.

originak code by
	NAME			STones80
	AUTHOR			bruce@utafll.uta.edu (Bruce Samuelson)
	FUNCTION		low and medium level benchmarks for ST80 and ST/V
	ST-VERSIONS		pre R4.0, R4.0, R4.1, ST/V
	PREREQUISITES	need floating point hardware or emulation
	CONFLICTS		none
	DISTRIBUTION	world
	VERSION			1.0
	DATE			April 16, 1993

!

!SMarkSmopstone class methodsFor: 'class initialization' stamp: ''!
initialize
	"added by nishis"
	"SmopstoneBenchmark initialize."

	| block |
	WhichFloatPrintOn := 7850 asFloat printString size = '7850.0' size.

	IsSupportRecursiveBlock := true.
	block := [:var | (var < 2) ifTrue: [block value: var + 1]].
	[block value: 1] 
		ifError: 
			[ :msg :rec | 
			IsSupportRecursiveBlock := false]
! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: ''!
benchStone
	"modified by nishis"
	| n nTests times stones printA printC param time0 expln block time stone geoMean gm power |
	
	n := self problemSize.	 "Each test is repeated this many times. The smopstone times in
			the test parameters are normalized to a value of one. You may
			set it to a higher number if your machine is really blazing."
	output
		cr;
		cr;
		nextPutAll: 'Starting benchmarks with repetition count = ' , n printString , '.'.
	nTests := testParams size.
	nTests = testBlocks size
		ifFalse: [ self error: 'Inconsistent test count.' ].
	times := OrderedCollection new.
	stones := OrderedCollection new.	"The following blocks are restricted to two args by ST/V-DOS."
	printA := [ :time1 :smop1 | 
	output cr.
	output nextPutAll: time1 printString.
	output nextPutAll: '     '.
	output nextPutAll: smop1 printString.
	output nextPutAll: '     ' ].
	printC := [ :expln1 | output nextPutAll: expln1 ].
	output
		nextPutAll:
			'

time in    smop-
seconds    stones    explanation
'.
	1 to: nTests do: [ :i | 
		param := testParams at: i.
		time0 := param at: 1.	"seconds for one-smopstone machine"
		expln := param at: 2.
		block := testBlocks at: i.
		time := Time millisecondsToRun: [ n timesRepeat: block ].
		time := (time max: 1) / 1000.0.	"time is now in seconds"
		stone := n * time0 / time.
		times add: time.
		stones add: stone.
		printA value: time value: stone.
		printC value: expln ].
	geoMean := [ :numbers | 
	gm := 1.
	power := 1 / nTests.
	numbers do: [ :number | gm := gm * (number raisedTo: power) ].
	gm ].
	output cr.
	printA value: (geoMean value: times) value: (geoMean value: stones).
	printC value: 'geometric mean'.
	IsSupportRecursiveBlock
		ifFalse: [ output nextPutAll: '*' ].
	output
		cr;
		cr;
		nextPutAll: 'Benchmarks complete.';
		cr.
	IsSupportRecursiveBlock
		ifFalse: [ 
			output
				cr;
				nextPutAll:
						'*** Caution ***
2 test of this benchmark (marked at ''*'') is intended to test the efficiency 
of recursively calling a block, that it isn''t supported by this VM.
So the result would be fairly pointless.';
				cr ]! !

!SMarkSmopstone methodsFor: 'defaults' stamp: ''!
defaultProblemSize
	^ 1! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'nishis 2/14/98 02:47'!
fractonacci: n 
	"Return something like the fibonacci function of n but
	using fractional numbers rather than whole ones. The
	reason for this variation is to run long enough to get
	a decent time measurement without exceeding 16383, the
	limit of small integers for ST/V-DOS. Choosing n = 13/2
	takes enough time and computes to 13581.

	Fibonacci uses n-1 and n-2 instead of n-(1/2) and n-(1/3).
	However, I couldn't get it to run in the above constraints.

	This benchmark tests the efficiency of recursively calling
	a method that does a little fractional arithmetic internally."

	^n > 1
		ifTrue: [(self fractonacci: n - (1/2)) + (self fractonacci: n - (1/3))]
		ifFalse: [1]! !

!SMarkSmopstone methodsFor: 'recursive method' stamp: ''!
generate: lastGen counts: counts
	"added by nishis"
	"SmopstoneBenchmark new sorcerersApprentice."

	| intersection isIncluded nextGen |
	
	counts add: lastGen size.
	nextGen := OrderedCollection new.
	lastGen do:
		[:r1 |
		lastGen do: 
			[:r2 | 
			(r1 origin ~= r2 origin or: [r1 corner ~= r2 corner])
			"In ST80 this test would have simply been r1 ~= r2"
				ifTrue:
					[(r1 intersects: r2)
						ifTrue:
							[intersection := r1 intersect: r2.
							isIncluded := false.                     "All these lines"
							nextGen do:                              "would have been"
								[:rec |                                "avoided if we"
									(rec origin = intersection origin and: "could have used"
									[rec corner = intersection corner])    "a set for"
										ifTrue: [isIncluded := true]].       "nextGen. See"
							isIncluded                               "explanation"
								ifFalse:                               "above."
									[nextGen size > 500
										ifTrue: [self error: 'Converges too slowly.']
										ifFalse: [nextGen add: intersection]]]]]].
		nextGen size > 0 ifTrue: [self generate: nextGen counts: counts].
	^counts! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: ''!
primesUpTo: n
	"modified by nishis"

	"Return the prime numbers between 2 and n.

	This method tests the efficiency of recursively calling a block
	that does some collection enumeration based on integer arithmetic."

	| nSqrt lowPrimes highPrimes genNext first nextPrime2 |
	n < 5 | (n > 16363) ifTrue: [self error: 'Prime limit(s) out of range.'].
	nSqrt := n sqrt rounded.
	lowPrimes := OrderedCollection with: 2.
	highPrimes := 5 to: n by: 2.
	IsSupportRecursiveBlock
		ifTrue:
			[genNext :=
				[:nextPrime |
				lowPrimes add: nextPrime.
				highPrimes := highPrimes select: [:k | k \\ nextPrime ~= 0].
				(first := highPrimes first) <= nSqrt ifTrue: [genNext value: first]].
			genNext value: 3.]
		ifFalse: 
			[first := highPrimes first.
			nextPrime2 := 3.
			[first <= nSqrt] 
				whileTrue: 
					[lowPrimes add: nextPrime2.
					highPrimes := highPrimes select: [:k | k \\ nextPrime2 ~= 0].
					(first := highPrimes first) <= nSqrt ifTrue: [nextPrime2 := first]]].
	^lowPrimes , highPrimes! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: ''!
problemSize
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'nishis 2/14/98 02:46'!
readme

	"INTRODUCTION 
	 
	Smopstone: Smalltalk Medium level OPeration Stones 
	Portable Medium level Benchmarks for ST80 and ST/V (using 16-bit 
	SmallInts) Placed in public domain January 1993  (c) Bruce Samuelson 
	Permission is given to place this in public Smalltalk archives 
	 
	Use monospaced fonts if possible to view the methods in this class. 
	 
	(1) Collect garbage if supported (2) do 'SmopstoneBenchmark new 
	runBenchmark'. Results are printed in the Transcript window. 
	Post results for your machines to comp.lang.smalltalk or 
	mail them to bruce@ling.uta.edu or bruce@utafll.uta.edu. 
	 
	DISCUSSION 
	    
	This readme method would normally be in the class comment for ST80. 
	ST/V-DOS doesn't support class comments. 
	 
	These benchmarks are a companion to the SlopstoneBenchmark class 
	posted to 
	comp.lang.smalltalk this month. Slopstones tested low level operations. 
	  
	Smopstones test medium level operations that exercise recursive block 
	and  
	method calls, collection building and enumeration, streaming, and 
	sorting. The 
	lower level operations contained in them exercise arithmetic (mostly 
	integer, with some fractions and floats) string manipulation, and low 
	level streaming. 
	 
	The benchmarks do not test applications. They also do not test user 
	interface performance because of the non-portability of this area of 
	Smalltalk and its  
	sensitivity to the speed of the video subsystem. The tests are cpu bound. 
	They do not access files and should not cause disk paging. 
	 
	The main weaknesses of the benchmarks are (1) they are not high 
	enough level 
	to test actual applications, and (2) they concentrate in too few areas of 
	Smalltalk, omitting many of the diverse capabilities of its class library. 
	My excuse is that one can only devote limited time writing public 
	domain benchmarks. 
	 
	The tests avoid generating integers larger than 16383, the maximum 
	SmallInteger in ST/V-DOS. 16-bit implementions would perform worse 
	with larger 
	integers. The benchmarks are also suitable for testing 32-bit versions of 
	Smalltalk. They try to avoid other pitfalls that would skew the results 
	such as the lack of an adequate hash function for a class. Someone 
	warned of this 
	in comp.lang.smalltalk (I forget who). 
	 
	DEFINITION OF REFERENCE MACHINE (ONE SMOPSTONE) 
	 
	The following machine is the one on which I developed these 
	benchmarks. By 
	convention it is defined to operate at one smopstone. It's a mid range 
	performer for current ParcPlace versions of Smalltalk. 
	 
	Hardware: Amax 486DX/33 (includes internal floating point processor 
	and internal 8K cache), 256K external cache, 16MB RAM. 
	 
	Software: ParcPlace VisualWorks 1.0, Windows 3.1, DOS 5.0 (plain vanilla 
	setup). 
	 
	COMPARISON TO XEROX DORADO 
	 
	For reference, the machine runs at 649% of a Dorado on ParcPlace 
	benchmarks for ST80 4.1. Its fast video card helps on these PPS 
	benchmarks. I didn't run 
	them for VisualWorks 1.0. It would be somewhat slower because there 
	are vastly 
	more classes. 
	 
	EXAMPLE RESULTS FOR REFERENCE MACHINE 
	 
	time in    smop- 
	seconds    stones    explanation 
	 
	 
	3.157      1.0       generating fractonaccis 
	1.123      1.0       generating primes 
	1.091      1.0       generating and parsing streams 
	3.091      1.0       generating strings 
	1.167      1.0       forming sets 
	5.139      1.0       sorting strings 
	5.601      1.0       sorcerer's apprentice 
	 
	2.355      1.0       geometric mean"
	^ self! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'nishis 2/14/98 02:46'!
setFrom: collection
	"Form a set from collection and return it.

	This method tests the efficiency of building a fairly large set
	from strings. It indirectly tests the effectiveness of the string
	hash function. Strings are used often enough as dictionary keys
	that this may be worth including in the benchmark suite.  ST/V-DOS
	has a primitive hash for strings, and ST80 has an elaborate one
	written in Smalltalk."

	^collection asSet! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: ''!
setUp
	"modified by nishis"

	"Numbers in testParams represent the approximate number of seconds it
	takes to run the tests for a one-smopstone machine.

	Numbers in testBlocks are parameters tuned for each test. Do not
	change them. The times for several tests depend on them non-linearly."

	| primes strings set |
	
	output := TextStream with: String new.

	testParams := OrderedCollection new.
 
	IsSupportRecursiveBlock 
		ifTrue: 
			[testParams
				add: #(3.157 'generating fractonaccis');
				add: #(1.123 'generating primes');
				add: #(1.091 'generating and parsing streams');
				add: #(3.091 'generating strings');
				add: #(1.167 'forming sets');
				add: #(5.139 'sorting strings');
				add: #(5.601 'sorcerer''s apprentice')] 
		ifFalse: 
			[testParams
				add: #(3.157 'generating fractonaccis');
				add: #(1.123 'generating primes*');
				add: #(1.091 'generating and parsing streams');
				add: #(3.091 'generating strings');
				add: #(1.167 'forming sets');
				add: #(5.139 'sorting strings');
				add: #(5.601 'sorcerer''s apprentice*')].

	testBlocks := OrderedCollection new.

	testBlocks
		add: [self fractonacci: 13/2];
		add: [primes := self primesUpTo: 9000];	"7351 - 1"
		add: [self streamTestsOn: primes];
		add: [strings := self stringsUpTo: 8000];
		add: [set := self setFrom: strings];
		add: [self sort: set];
		add: [self sorcerersApprentice]! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: ''!
sorcerersApprentice
	"modified by nishis"

	" FORMATTED FOR MONOSPACED FONT

	Perform various operations on rectangles.

	This method tests the efficiency of recursively calling a block that
	includes lots of integer arithmetic, collection building, and collection
	enumeration. The method:

	(1) Creates a collection of pseudo random rectangles 
	(2) Forms a new collection of all their intersections 
	(3) Recursively continues until there are no more intersections 
	(4) Returns a collection with the counts of rectangles in each generation.

	Because the intersections are forming progressively smaller rectangles
	(we exclude intersections of a rectangle with itself), the algorithm will
	eventually converge. Depending on the choice of numeric parameters, it may
	converge very quickly or very slowly. The parameters used below make it
	converge in a reasonable amount of time (a few seconds on a one-smopstone
	machine). It took some experimentation with different combinations to
	achieve this.

	The pseudo random number generator isn't very good, but it's adequate
	for this benchmark. I had intended the number '87' it uses to be a prime,
	but 87 = 29 * 3. The numbers may have been a bit more random otherwise.

	One could write an algorithm that would converge much more quickly and in
	a more predictable amount of time by sorting the intermediate rectangles
	in two dimensions and not bothering to test for intersections those
	rectangles that are contained in mutually exclusive regions. We have
	chosen algorithmic simplicity over performance optimization. We simply
	perform intersections of each rectangle with every possible partner in
	each generation. The time consumed is quadratic in the number of rectangles.

	The algorithm originally stored rectangles in sets to eliminate duplicates.
	Unfortunately, ST/V-DOS uses the hash function inherited from Object for
	Rectangle, which will allow duplicates to be stored. So we were forced to
	store rectangles in ordered collections and eliminate duplicates by brute
	force. The brutality was heightened because we could not use the test
	collection>>includes: to decide whether to add a rectangle to the ordered
	collections, since ST/V-DOS does not define equality (=) for rectangles
	either. The remaining warts in the code are not worth explaining.

	In an actual application, these shortcomings of ST/V-DOS would have been
	overcome by adding subclasses and methods rather than writing kludgy code."

	| m n firstGen intersection isIncluded counts r random
	a b c d e f g h generate nextGen |
	
	m := 80.
	n := 20 * m.
	firstGen := OrderedCollection new.
	counts := OrderedCollection new.
	r := 50.
	random := [r := r + 1 * 87 \\ n].
	m timesRepeat: [
		a := random value.
		b := random value.
		c := random value.
		d := random value.
		e := a min: b.
		f := c min: d.
		g := a max: b.
		h := c max: d.
		firstGen add: (Rectangle origin: e @ f corner: g @ h)].
	generate := 
		[:lastGen |
		counts add: lastGen size.
		nextGen := OrderedCollection new.
		lastGen do:
			[:r1 |
			lastGen do: 
				[:r2 | 
				(r1 origin ~= r2 origin or: [r1 corner ~= r2 corner])
				"In ST80 this test would have simply been r1 ~= r2"
					ifTrue:
						[(r1 intersects: r2)
							ifTrue:
								[intersection := r1 intersect: r2.
								isIncluded := false.                     "All these lines"
								nextGen do:                              "would have been"
									[:rec |                                "avoided if we"
										(rec origin = intersection origin and: "could have used"
										[rec corner = intersection corner])    "a set for"
											ifTrue: [isIncluded := true]].       "nextGen. See"
								isIncluded                               "explanation"
									ifFalse:                               "above."
										[nextGen size > 500
											ifTrue: [self error: 'Converges too slowly.']
											ifFalse: [nextGen add: intersection]]]]]].
			nextGen size > 0 ifTrue: [generate value: nextGen]].
	IsSupportRecursiveBlock
		ifTrue: [generate value: firstGen]
		ifFalse: [^ self generate: firstGen counts: counts].
	^counts! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'nishis 2/14/98 02:39'!
sort: collection
	"Form a sorted collection from collection and return it.

	This method tests the efficiency of sorting a fairly large
	collection of strings. It indirectly measures the efficiency
	of the sorting algorithm and of string comparison operations."

	^collection asSortedCollection! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'sm 1/17/2011 09:55'!
streamTestsOn: integers 
	"modified by nishis"
	"Test steaming operations on the collection of integers. 
	 
	This method measures the efficiency of integer-to-float conversion, of 
	printing numbers to a write stream, of parsing tokens in a read 
	stream,   and of converting the tokens from strings to numbers. The 
	technique for 
	converting tokens into floats is constrained by portability between 
	ST80 and ST/V. 
	 
	To validate the logic, the original integers are compared with the 
	final   floats. There should be no roundoff errors."
	"The following line accounts for the different implementations of 
	Float>>printString for some versions of Smalltalk. USA versions use 
	the decimal character, while some European versions use the comma 
	char.   Thanks to Marten Feldtmann for pointing this out."

	| delim space s floats float string |
	WhichFloatPrintOn 
		ifTrue: [delim := 1.0 printString at: 2]	"$. for USA, $, for some Europe."
		ifFalse: [delim := $e].					"modified by nishis"

	space := Character value: 32.
	"Can't use Character space in ST/V-DOS"
	s := ReadWriteStream on: String new.
	integers do: 
		[:i | 
		WhichFloatPrintOn 
			ifTrue: [i asFloat printOn: s]
			ifFalse: [i asFloat testPrintOn: s base: 10].
		s space].
	"Now make sure the underlying string size is < 16383, a 16-bit small int."
	s contents size > (SmallInteger maxVal) ifTrue: [self error: 'String too big.'].
	s reset.
	floats := OrderedCollection new: integers size.
	[s atEnd]
		whileFalse: 
			[float := 0.0.
			string := s upTo: delim.
			s upTo: space.
			"In the following, digitValue is portable between ST80 and ST/V-DOS. "
			string do: [:char | float := float * 10.0 + char digitValue].
			floats add: float].
	integers = floats ifFalse: [self error: 'Numbers do not compare.']! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'nishis 2/14/98 02:39'!
stringsUpTo: n 
	"Return a collection of strings representing the integers from 1 
	to n with their digits reversed. 
	 
	This method tests the efficiency of creating small streams, performing 
	string operations, and building collections. It includes a gross kludge 
	to coerce portability between ST80 and ST/V. They vary slightly in the 
	selector used to reverse collections. Subsequent to freezing this code, 
	someone pointed out that it could have been written more simply 
	without   using perform:. It could also have been simplified using 
	select:. It 
	was too late to change. The techniques are left as an exercise for the 
	reader."

	| selector |
	(Array with: #reverse with: #reversed)
		do: [:symbol | (String canUnderstand: symbol)
			ifTrue: [selector := symbol]].
	^ (1 to: n)
		collect: [:m | m printString perform: selector]! !

!SMarkSuite class methodsFor: 'defaults' stamp: ''!
defaultProblemSize
	^ nil! !

!SMarkSuite class methodsFor: 'defaults' stamp: ''!
defaultRunner
	^ SMarkRunner! !

!SMarkSuite class methodsFor: 'benchmarking' stamp: ''!
isAbstractClass
	"This is a hack that is necessary in Squeak since it does not provide #isAbstractClass.
	 Actually this class is supposed to be abstract, but well, inheritance..."
	
	^ false! !

!SMarkSuite class methodsFor: 'benchmarking' stamp: ''!
run
	^ self defaultRunner execute: self new.! !

!SMarkSuite class methodsFor: 'benchmarking' stamp: ''!
run: nIterations
	^ self defaultRunner execute: self new with: nIterations.! !

!SMarkSuite methodsFor: 'helper' stamp: ''!
benchmarkNameForSelector: selector
	"Extracts the actual name of the benchmark from the selector"
	(selector beginsWith: #bench) ifTrue: [ ^ selector copyFrom: 6 to: selector size].
	^ selector asSymbol
	! !

!SMarkSuite methodsFor: 'running' stamp: ''!
cleanUpInstanceVariables
	"Make sure all variables that are 'user variables' get cleaned"
	
	self class allInstVarNames do: [ :name |
		name = 'runner' ifFalse: [
			self instVarNamed: name put: nil ] ]! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
performCustomSelector: aSelector with: aPrefix
	| customSelector |
	customSelector := (aPrefix, aSelector capitalized) asSymbol.
	(self respondsTo: customSelector) ifTrue: [ 
		self perform: customSelector].! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
problemSize
	^ runner problemSize! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
run
	"Executes all the benchmarks in the suite, 
	 coordinating with the runner when necessary"
	
	| potentialBenchmarkSelectors |
	selectedBenchmarks
		ifNotNil: [ potentialBenchmarkSelectors := selectedBenchmarks ]
		ifNil:    [ potentialBenchmarkSelectors := self class allSelectors ].				
	
	potentialBenchmarkSelectors
		do: [ :selector |
			(self shouldRunSelector: selector)
				ifTrue: [
					runner performBenchmark: selector ]
		].
	! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
runBenchmark: aSelector
	
	[self setUp.
	 self performCustomSelector: aSelector with: #setUp.
	 runner timedBenchmarkExecution: aSelector] ensure: [
		self performCustomSelector: aSelector with: #tearDown.
		self tearDown.
		self cleanUpInstanceVariables]! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
runOnly: aSymbol
	selectedBenchmarks := IdentitySet newFrom: { aSymbol }.! !

!SMarkSuite methodsFor: 'accessing' stamp: ''!
runner
	^ runner! !

!SMarkSuite methodsFor: 'accessing' stamp: ''!
runner: aRunner
	runner := aRunner.! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
selectedBenchmarks
	^ selectedBenchmarks! !

!SMarkSuite methodsFor: 'running' stamp: ''!
setUp
	"subclass responsibility to set up the necessary environment for a benchmark"
	^ self! !

!SMarkSuite methodsFor: 'helper' stamp: ''!
shouldRunSelector: selector
	(selector includes: $:) ifTrue: [ ^ false ].
	^ selector beginsWith: #bench! !

!SMarkSuite methodsFor: 'running' stamp: ''!
tearDown
	"subclass responsibility to clean up the environment after a benchmark"
	^ self! !

SMarkSuite subclass: #SMarkTestRunnerSuite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestRunnerSuite commentStamp: 'StefanMarr 3/13/2011 22:58' prior: 0!
This is a benchmark suite used to test the benchmark runner!

!SMarkTestRunnerSuite class methodsFor: 'as yet unclassified' stamp: ''!
isAbstractClass
	"This is more like a hack, but we do not want to execute this as a benchmark"
	^ true! !

!SMarkTestRunnerSuite methodsFor: 'as yet unclassified' stamp: ''!
benchDoNothing
	"Just a dummy that should be executed and result in some measured value"
	^ self! !

!SMarkTestRunnerSuite methodsFor: 'as yet unclassified' stamp: ''!
benchNotifyRunner
	runner notifyRunnerWasExecuted.
	^ self! !

SMarkSuite subclass: #SMarkTestRunnerSuiteForAutosizing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestRunnerSuiteForAutosizing class methodsFor: 'as yet unclassified' stamp: ''!
defaultRunner
	^ SMarkTestRunner! !

!SMarkTestRunnerSuiteForAutosizing methodsFor: 'as yet unclassified' stamp: ''!
baseBenchmark
	(Delay forMilliseconds: 5) wait.! !

!SMarkTestRunnerSuiteForAutosizing methodsFor: 'as yet unclassified' stamp: ''!
benchBig
	(Delay forMilliseconds: 100) wait.! !

!SMarkTestRunnerSuiteForAutosizing methodsFor: 'as yet unclassified' stamp: ''!
benchSmall
	(Delay forMilliseconds: 5) wait.! !

SMarkSuite subclass: #SMarkTestRunnerSuiteForPerfromBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestRunnerSuiteForPerfromBenchmark class methodsFor: 'as yet unclassified' stamp: ''!
defaultRunner
	^ SMarkTestRunner! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
BenchShouldNotBeExecuted
	"This one should not be executed"
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #BenchShouldNotBeExecuted.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
baseBenchmark
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #baseBenchmark.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
benchAfsdfsdfsdf
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #benchAfsdfsdfsdf.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
benchBsdfasereSDSfsdfsdfs
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #benchBsdfasereSDSfsdfsdfs.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
benchC
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #benchC.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
benchZZ
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #benchZZ.! !

SMarkSuite subclass: #SMarkTestSuite
	instanceVariableNames: 'a b c d'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestSuite commentStamp: 'StefanMarr 3/12/2011 22:13' prior: 0!
This class is used for unit testing only.!

!SMarkTestSuite class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/20/2011 21:09'!
defaultRunner
	^ SMarkTestRunner! !

!SMarkTestSuite class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/13/2011 20:57'!
isAbstractClass
	"This is more like a hack, but we do not want to execute this as a benchmark"
	^ true! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:15'!
a
	^a! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:15'!
b
	^b! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 00:35'!
benchA
	a := #executed.
	(Delay forMilliseconds: 1) wait.! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 00:38'!
benchB
	b := #executed.
	(Delay forMilliseconds: 1) wait.! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:37'!
benchC
	c := #executed.! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 01:04'!
benchWithMultipleCriteria
	| t1 t2 t3 |
	t3 := runner createTimer: 't3'.
	t3 start.
	
	(Delay forMilliseconds: 25) wait.
	
	t2:= runner createTimer: 't2'.
	t2 start.
	(Delay forMilliseconds: 15) wait.	
	
	t1 := runner createTimer: 't1'.  
	t1 start.
	(Delay forMilliseconds: 10) wait.
	
	
	t3 stop.
	t2 stop.
	t1 stop.    ! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:15'!
c
	^c! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:15'!
d
	^d! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:50'!
initialize
	super initialize.
	
	"Filling in some test data"
	a := 1.
	b := 2.
	c := #three.
	d := '42'.
	! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:39'!
setUp
	runner countSetUpInvoke! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/13/2011 20:29'!
setUpBenchB
	runner didSetUpBenchB! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:39'!
tearDown
	runner countTearDownInvoke! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/13/2011 20:30'!
tearDownBenchC
	runner didTearDownBenchC! !

SMarkTestSuite subclass: #SMarkTestSuiteWithBaseBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestSuiteWithBaseBenchmark methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 18:32'!
baseBenchmark
	runner didRunBaseBenchmark.! !

Object subclass: #SMarkTimer
	instanceVariableNames: 'startTime elapsedTime name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkTimer class methodsFor: 'instance creation' stamp: 'StefanMarr 3/14/2011 08:54'!
new: aName
	| timer |
	
	timer := super new.
	timer name: aName.
	
	^timer! !

!SMarkTimer methodsFor: 'timing' stamp: 'StefanMarr 3/20/2011 12:24'!
currentMillis
	"hack to make it work under most Smalltalkses and Pinocchio"
	"STEFAN: had to take this out, since my images do not support pragmas
	 <pPrimitive: #millisecondClock plugin: #'Chronology.Time' >"
	^ Time millisecondClockValue! !

!SMarkTimer methodsFor: 'initialization' stamp: 'StefanMarr 3/14/2011 08:54'!
initialize
	elapsedTime := 0! !

!SMarkTimer methodsFor: 'accessing' stamp: 'StefanMarr 3/14/2011 08:54'!
name
	^name! !

!SMarkTimer methodsFor: 'accessing' stamp: 'StefanMarr 3/14/2011 08:54'!
name: aString
	name := aString ! !

!SMarkTimer methodsFor: 'timing' stamp: 'StefanMarr 3/14/2011 08:54'!
reset
	startTime := 0.
	elapsedTime := 0.! !

!SMarkTimer methodsFor: 'timing' stamp: 'StefanMarr 3/20/2011 12:24'!
start
	startTime := self currentMillis.! !

!SMarkTimer methodsFor: 'timing' stamp: 'StefanMarr 3/20/2011 12:25'!
stop
	| elapsedInThisPeriod current |
	current := self currentMillis.
	
	elapsedInThisPeriod := Time milliseconds: current since: startTime.
	
	elapsedTime := elapsedTime + elapsedInThisPeriod.! !

!SMarkTimer methodsFor: 'accessing' stamp: 'StefanMarr 3/14/2011 08:54'!
totalTime
	^elapsedTime! !

Object subclass: #SMarkTransporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkTransporter commentStamp: 'sm 5/25/2011 14:09' prior: 0!
SMarkTransporter is not actually a Transporter class, since there are currently no needs for customization.
Thus, it is just a dummy class for future use, and to hold #transportersForFileOutMenu.
!

!SMarkTransporter class methodsFor: 'transporter' stamp: 'sm 5/25/2011 14:08'!
transportersForFileOutMenu
	^ { Transporter forPackage: (PackageInfo named: 'SMark') }! !

Object subclass: #SMarkWeakScalingRunnerExecutor
	instanceVariableNames: 'numInnerIterations benchmarkSelector suite runner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkWeakScalingRunnerExecutor class methodsFor: 'as yet unclassified' stamp: 'sm 5/30/2011 16:33'!
createFor: aSelector for: numIterations with: aRunner and: aSuite
	| o |
	o := self new.
	o runner: aRunner.
	o suite: aSuite.
	o innerIterations: numIterations.
	o benchmarkSelector: aSelector.
	^ ([ o run ] newProcess)! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'accessing' stamp: 'sm 5/30/2011 16:20'!
benchmarkSelector: aSelector
	benchmarkSelector := aSelector! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'accessing' stamp: 'sm 5/30/2011 16:20'!
innerIterations: anInt
	numInnerIterations := anInt! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'benchmarking' stamp: 'sm 5/30/2011 16:22'!
run
	1 to: numInnerIterations do: [:i |
		suite perform: benchmarkSelector.].
	
	runner executorCompleted.! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'accessing' stamp: 'sm 5/30/2011 16:21'!
runner: aRunner
	runner := aRunner! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'accessing' stamp: 'sm 5/30/2011 16:21'!
suite: aSuite
	suite := aSuite! !
SMarkSmopstone initialize!
